Package: FrailtyCompRisk
Title: Competing Risks Models for Multi-Center Survival Data with
        Frailty
Version: 0.1.1
Authors@R: c(person("Benjamin", "Delmas", email = "benjamin.delmas@ensae.fr", role = "aut"),
    person("Lucas", "Ducrot", email = "lucas.ducrot@inria.fr", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8020-2897")),
    person("Sandrine", "Katsahian", email = "sandrine.katsahian@aphp.fr", role = "aut",
           comment = c(ORCID = "0000-0002-7261-0671")),
    person(given = "Inria", role = "cph"))
Description: Implements methods for analyzing competing risks data in
    multi-center survival studies using frailty models. The approach relies on
    a mixed proportional hazards model for the sub-distribution, allowing for
    cluster-specific random effects. The package provides tools for model
    estimation with or without frailty using Maximum Likelihood (ML) and
    Restricted Maximum Likelihood (REML). It supports flexible modeling of
    between-center heterogeneity and is particularly suited for multi-center
    clinical trials or registries. Core features include data simulation,
    likelihood computation, cluster-dependent censoring options, and testing of
    frailty effects. For methodological details, see Katsahian et al. (2006) <doi:10.1002/sim.2684>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: Matrix, stats
Suggests: dplyr, tibble, kableExtra, knitr, rmarkdown, testthat (>=
        3.0.0)
VignetteBuilder: knitr
URL: https://github.com/TeamHeKA/FrailtyCompRisk
BugReports: https://github.com/TeamHeKA/FrailtyCompRisk/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-23 10:32:04 UTC; utilisateur
Author: Benjamin Delmas [aut],
  Lucas Ducrot [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8020-2897>),
  Sandrine Katsahian [aut] (ORCID:
    <https://orcid.org/0000-0002-7261-0671>),
  Inria [cph]
Maintainer: Lucas Ducrot <lucas.ducrot@inria.fr>
Repository: CRAN
Date/Publication: 2026-04-27 14:20:09 UTC
Built: R 4.5.2; ; 2026-05-01 17:37:17 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Ml_CompRisk)
export(Ml_Cox)
export(Parameters_estimation)
export(Reml_CompRisk_frailty)
export(Reml_Cox_frailty)
export(check_data_format)
export(simulate_data)
importFrom(Matrix,Diagonal)
importFrom(Matrix,Matrix)
importFrom(Matrix,sparseMatrix)
importFrom(stats,pchisq)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,uniroot)

---
editor_options: 
  markdown: 
    wrap: 72
---

# FrailtyCompRisk

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/FrailtyCompRisk)](https://CRAN.R-project.org/package=FrailtyCompRisk)
[![R-CMD-check](https://github.com/TeamHeKA/FrailtyCompRisk/actions/workflows/r.yml/badge.svg)](https://github.com/TeamHeKA/FrailtyCompRisk/actions/workflows/r.yml)
[![License:
GPLv3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)

<!-- badges: end -->

## 📚 Overview

The goal of **`FrailtyCompRisk`** is to provide tools for **competing
risks survival analysis** in **multicenter studies**, accounting for
**unobserved heterogeneity** (frailty) at the center level as proposed
in the following article:

Katsahian S, Resche-Rigon M, Chevret S, Porcher R. Analysing multicentre
competing risks data with a mixed proportional hazards model for the
subdistribution. *Stat Med*. 2006 Dec 30;25(24):4267-78. doi:
[10.1002/sim.2684](https://onlinelibrary.wiley.com/doi/10.1002/sim.2684).
PMID: [16960919.](https://pubmed.ncbi.nlm.nih.gov/16960919/)

This package is particularly useful for statisticians and
epidemiologists analyzing time-to-event data where individuals are
nested within centers (e.g., hospitals or clinics), and where multiple
causes of failure may occur.

It includes functions to: - Simulate clustered competing risks data, -
Fit cause-specific Cox models with shared frailty, following the method
presented by Sandrine Katsahian in "Analysing multicentre competing
risks data with a mixed proportional hazards model for the
subdistribution." - Fit cause-specific Cox models, - Fit Cox models with
shared frailty, - Fit Cox models.

The REML method is used when shared frailty is modeled; otherwise, the
ML method is applied.

## Results

We conducted the same simulation-based tests as in "Analysing
multicentre competing risks data with a mixed proportional hazards model
for the subdistribution.", and we obtained similar results:

| N   | K   | θ   | θ̂     | MSE(θ̂)  | γ       | γ̂        | MSE(γ̂)    |
|-----|-----|-----|-------|---------|---------|----------|-----------|
| 200 | 5   | 0.0 | 0.024 | 0.00244 | (0)     | (-0.068) | (0.23858) |
| 200 | 20  | 0.0 | 0.055 | 0.01027 | (0)     | (0.06)   | (0.34056) |
| 500 | 5   | 0.0 | 0.012 | 0.00062 | (0)     | (-0.001) | (0.10598) |
| 500 | 20  | 0.0 | 0.020 | 0.00137 | (0)     | (0.004)  | (0.0921)  |
| 200 | 5   | 0.0 | 0.036 | 0.00690 | (0.375) | (0.345)  | (0.23996) |
| 200 | 20  | 0.0 | 0.054 | 0.01221 | (0.375) | (0.381)  | (0.29773) |
| 500 | 5   | 0.0 | 0.012 | 0.00068 | (0.375) | (0.354)  | (0.10479) |
| 500 | 20  | 0.0 | 0.020 | 0.00161 | (0.375) | (0.42)   | (0.10242) |
| 200 | 5   | 0.1 | 0.130 | 0.03940 | (0)     | (-0.069) | (0.25903) |
| 200 | 20  | 0.1 | 0.127 | 0.02099 | (0)     | (-0.025) | (0.31313) |
| 500 | 5   | 0.1 | 0.108 | 0.01366 | (0)     | (-0.013) | (0.0985)  |
| 500 | 20  | 0.1 | 0.106 | 0.00754 | (0)     | (-0.056) | (0.10335) |
| 200 | 5   | 0.1 | 0.127 | 0.02148 | (0.375) | (0.372)  | (0.26376) |
| 200 | 20  | 0.1 | 0.139 | 0.02608 | (0.375) | (0.365)  | (0.27877) |
| 500 | 5   | 0.1 | 0.111 | 0.01158 | (0.375) | (0.378)  | (0.10493) |
| 500 | 20  | 0.1 | 0.099 | 0.00713 | (0.375) | (0.377)  | (0.10856) |
| 200 | 5   | 0.6 | 0.655 | 0.43517 | (0)     | (0.035)  | (0.22487) |
| 200 | 20  | 0.6 | 0.541 | 0.11904 | (0)     | (-0.009) | (0.35059) |
| 500 | 5   | 0.6 | 0.611 | 0.29234 | (0)     | (0.001)  | (0.10746) |
| 500 | 20  | 0.6 | 0.598 | 0.06516 | (0)     | (0.004)  | (0.10813) |
| 200 | 5   | 0.6 | 0.649 | 0.41894 | (0.375) | (0.398)  | (0.2656)  |
| 200 | 20  | 0.6 | 0.570 | 0.11168 | (0.375) | (0.378)  | (0.25024) |
| 500 | 5   | 0.6 | 0.629 | 0.30846 | (0.375) | (0.381)  | (0.10446) |
| 500 | 20  | 0.6 | 0.569 | 0.07018 | (0.375) | (0.38)   | (0.09621) |

## 📦 Installation

You can install the development version of `FrailtyCompRisk` from GitHub
using either [`pak`](https://pak.r-lib.org/) or
[`devtools`](https://github.com/r-lib/devtools):

``` r
# Using pak (recommended)
# install.packages("pak")
pak::pak("TeamHeKA/FrailtyCompRisk")

# Or using devtools
# install.packages("devtools")
devtools::install_github("TeamHeKA/FrailtyCompRisk")
```

## 📌 Usage

### 🧪 Data simulation

You can generate simple datasets following the competing risks model
with center effect frailty with the code below :

``` r
library(FrailtyCompRisk)

## SIZE AND CENTER REPARTITION
n_per_cluster=20 #sample size of each center
n_cluster=5 #number of centers
n=n_per_cluster * n_cluster #sample size
G=rep(1:n_cluster, each = n_per_cluster) #center repartition

## COVARIATES AND COMPETING FAILURE TIMES
n_cov=5 #number of covariables
Z=matrix(rnorm(n*n_cov,0,1),ncol = n_cov) #standardized Gaussian covariables 
prop=0.6 #proportion of failure 1
beta=c(1,1.2,0,-0.5,-0.2) #effects of covariates on failure cause 1
theta=0.6 #Variance of center effect frailty

## CENSORING
cens=TRUE #censoring in the simaluted data
pcens=0.25 #proportion of censoring
tau=1 #Variance of center effect on censoring

## DATA SIMULATION
data<-simulate_data(G,Z,prop,beta,theta,cens,pcens,tau)
```

### 📈 Parameters estimation

You can estimate the parameters of the model by using the function
**Parameters_estimation**.

You must specify the method: - Competing risks with shared frailty for
cause 1 (`"CompRisk_frailty"`) - Competing risks without frailty for
cause 1 (`"CompRisk"`) - Standard Cox model with shared frailty
(`"Cox_frailty"`) - Standard Cox model without frailty (`"Cox"`)

You must submit a data frame with at least 3 columns: - times: the
observed time, - status: 0 for right-censoring, i in [1,n] otherwise,
where i is the cause of failure, - clusters: cluster/group indicator. If
there is no cluster, clusters=rep(1,n), with n the size sample.

You also have to specify whether you want to take into account for a
center effect on censoring (cluster_censoring), if you chose
`method = "CompRisk_frailty"`. You may also specify a threshold of
convergence, a threshold for the variance of center effect frailty (in
the case it tends to zero and it might be negligible, hence the
threshold) and a maximum number of iterations.

``` r
method="CompRisk_frailty" #specify the method needed
cluster_censoring=F #center effect on censoring
max_iter=300 #maximum number of iterations
tol=1e-6 #threshold of convergence
threshold=1e-6 #threshold for the variance of center effect frailty

results <- Parameters_estimation(data,method,cluster_censoring,max_iter,tol,threshold)

results$beta #estimation of effects of covariates on failure cause 1
results$theta #(only for method "CompRisk_frailty" and "Cox_frailty") estimation of the variance of center effect frailty
results$u #(only for method "CompRisk_frailty" and "Cox_frailty") estimation of center effect frailty
results$p_value #(only for method "CompRisk_frailty" and "Cox_frailty") p_value for the test theta = 0, if p>0.05 or p is NA, it suggests that the cluster effect may be negligible.
```

### Complexity

The complexity of this algorithm is O(N² + Np + NK + (p + K)³)

where: - N: sample size - T: the number of iterations made, T \<
max_iter, where max_iter is a parameter you can choose - p: the number
of covariables - K: the number of clusters

This package uses sparse matrices with the package `Matrix`, which
improve significantly the speed of the algorithms.

## 🧠 Example on real data

This is under development...

## 📬 Contact

For any questions, issues, or suggestions, please open an issue on
GitHub.

## 🙏 Acknowledgments

Thanks to Lucas Ducrot and Benjamin Delmas for their contribution.

## 🧩 References

The `Matrix` package.

If you use this package in your research, please cite the following
article:

Katsahian S, Resche-Rigon M, Chevret S, Porcher R. Analysing multicentre
competing risks data with a mixed proportional hazards model for the
subdistribution. *Stat Med*. 2006 Dec 30;25(24):4267-78. doi:
[10.1002/sim.2684](https://onlinelibrary.wiley.com/doi/10.1002/sim.2684).
PMID: [16960919.](https://pubmed.ncbi.nlm.nih.gov/16960919/)

#' Unified Interface for Parameter Estimation in (Competing Risks) Frailty Models
#'
#' Provides a unified wrapper to estimate model parameters under four different modeling strategies:
#' \itemize{
#'   \item Competing risks with shared frailty for cause 1 (`"CompRisk_frailty"`)
#'   \item Competing risks without frailty for cause 1 (`"CompRisk"`)
#'   \item Standard Cox model with shared frailty (`"Cox_frailty"`)
#'   \item Standard Cox model without frailty (`"Cox"`)
#' }
#'
#' @param data A data frame containing at least the following columns:
#' \describe{
#'   \item{times}{Event or censoring times.}
#'   \item{status}{Event indicator: 0 = censored, 1 = event of interest, >1 = other types (for competing risks).}
#'   \item{clusters}{Cluster/group variable (e.g., center ID).}
#'   \item{covariates}{(Optional) Additional covariates from the 4th column onward.}
#' }
#' @param method Character string indicating the estimation method. One of:
#' \itemize{
#'   \item \code{"CompRisk_frailty"} (default): Competing risks model for cause 1 with shared frailty.
#'   \item \code{"CompRisk"}: Competing risks model for cause 1 without frailty.
#'   \item \code{"Cox_frailty"}: Cox proportional hazards model with shared frailty.
#'   \item \code{"Cox"}: Standard Cox proportional hazards model.
#' }
#' @param cluster_censoring Logical. If \code{TRUE}, adjusts for cluster-specific censoring. Only applicable when \code{method = "CompRisk_frailty"} (default = \code{FALSE}).
#' @param max_iter Maximum number of iterations for the selected model estimation (default = 300).
#' @param tol Convergence tolerance (default = 1e-6).
#' @param threshold Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).

#'
#' @return A named list whose structure depends on `method`.
#' For `"CompRisk"` and `"Cox"`, the returned list contains:
#' \describe{
#'   \item{beta}{Numeric vector of estimated regression coefficients.}
#' }
#' For `"CompRisk_frailty"` and `"Cox_frailty"`, the returned list may contain:
#' \describe{
#'   \item{beta}{Numeric vector of estimated fixed-effect coefficients.}
#'   \item{u}{Numeric vector of estimated cluster-specific frailties.}
#'   \item{theta}{Estimated frailty variance parameter.}
#'   \item{p_value}{P-value for testing the null hypothesis of no frailty effect.}
#' }
#' Returns `NULL` if estimation fails.
#'
#' @details
#' This wrapper allows seamless switching between various modeling frameworks. It automatically calls:
#' \itemize{
#'   \item \code{\link{Reml_CompRisk_frailty}} for `"CompRisk_frailty"`
#'   \item \code{\link{Ml_CompRisk}} for `"CompRisk"`
#'   \item \code{\link{Reml_Cox_frailty}} for `"Cox_frailty"`
#'   \item \code{\link{Ml_Cox}} for `"Cox"`
#' }
#'
#' Data format is validated via the helper function \code{\link{check_data_format}}. An error is raised if input format is invalid or if `cluster_censoring` is used with an incompatible method.
#'
#' @examples
#' \donttest{
#' data <- data.frame(
#'   times = c(4, 6, 10, 12, 3),
#'   status = c(1, 0, 2, 1, 0),
#'   clusters = c(1, 1, 2, 2, 3),
#'   x1 = rnorm(5),
#'   x2 = sample(0:1, 5, replace = TRUE)
#' )
#' result <- Parameters_estimation(data, method = "CompRisk_frailty")
#' result$beta
#' }
#'
#' @seealso \code{\link{Reml_CompRisk_frailty}}, \code{\link{Ml_CompRisk}}, \code{\link{Reml_Cox_frailty}}, \code{\link{Ml_Cox}}, \code{\link{check_data_format}}
#'
#' @export

Parameters_estimation <- function(data,method = "CompRisk_frailty",cluster_censoring = FALSE,max_iter = 300, tol = 1e-6,threshold = 1e-6)
{
  if (!(method %in% c("CompRisk_frailty","CompRisk","Cox_frailty","Cox")))
  {
    stop("The method chosen must be : 'CompRisk_frailty','CompRisk','Cox_frailty' or 'Cox'.")
  }
  if (!(check_data_format(data)) == TRUE)
  {
    stop()
  }
  if (cluster_censoring && (method != "CompRisk_frailty"))
  {stop("'cluster_censoring' is only for 'CompRisk_frailty' method.")}

  if (method == "CompRisk_frailty")
  {
    res = tryCatch(Reml_CompRisk_frailty(data,cluster_censoring,max_iter = 300, tol = 1e-6,threshold = 1e-6),
                   error = function(e) {
                     message(e$message)
                     return(NULL)})
  }
  if (method == "CompRisk")
  {
    res = tryCatch(Ml_CompRisk(data,max_iter = 300, tol = 1e-6),
                   error = function(e) {
                     message(e$message)
                     return(NULL)})
  }
  if (method == "Cox_frailty")
  {
    res = tryCatch(Reml_Cox_frailty(data,max_iter = 300, tol = 1e-6),
                   error = function(e) {
                     message(e$message)
                     return(NULL)})
  }
  if (method == "Cox")
  {
    res = tryCatch(Ml_Cox(data,max_iter = 300, tol = 1e-6),
                   error = function(e) {
                     message(e$message)
                     return(NULL)})
  }
  return(res)
}

test_that("Parameters_estimation needs a data.frame", {
  expect_error(Parameters_estimation(1,"Clearly a wrong method"),"The method chosen must be : 'CompRisk_frailty','CompRisk','Cox_frailty' or 'Cox'.")
})

test_that("Parameters_estimation needs a well structured data.frame.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  df_bis = data.frame(status = df$status,clusters = df$clusters,as.matrix(df[,4:length(df[1,])]))
  expect_error(Parameters_estimation(df_bis),"")
})

test_that("cluster_censoring is specific to the 'CompRisk_frailty' method.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  expect_error(Parameters_estimation(df,"CompRisk",cluster_censoring = TRUE),"'cluster_censoring' is only for 'CompRisk_frailty' method.")
})

test_that("The method 'CompRisk_frailty' works.",{
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  res = Parameters_estimation(df,"CompRisk_frailty")
  expect_equal(length(res$beta),n_cov)
  expect_equal(length(res$theta),1)
  expect_equal(length(res$u),n_cluster)
  expect_lte(abs(res$beta[1]-1),1)
  expect_lte(abs(res$beta[2]-1.2),1)
  expect_lte(abs(res$theta - 0.3),1)
  })
})

test_that("The method 'CompRisk' works.",{
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  res = Parameters_estimation(df,"CompRisk")
  expect_equal(length(res$beta), n_cov)
  expect_lte(abs(res$beta[1] - 1), 1)
  expect_lte(abs(res$beta[2] - 1.2), 1)
  })
})

test_that("The method 'Cox_frailty' works.",{
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n * n_cov, 0, 1), ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1, 1.2),theta = 0.6,cens = TRUE)
  res = Parameters_estimation(df, "Cox_frailty")
  expect_equal(length(res$beta), n_cov)
  expect_equal(length(res$theta), 1)
  expect_equal(length(res$u), n_cluster)
  expect_lte(abs(res$beta[1] - 1), 1)
  expect_lte(abs(res$beta[2] - 1.2), 1)
  expect_lte(abs(res$theta - 0.3), 1)
  })
})

test_that("The method 'Cox' works.",{
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  res = Parameters_estimation(df,"Cox")
  expect_equal(length(res$beta),n_cov)
  expect_lte(abs(res$beta[1]-1),1)
  expect_lte(abs(res$beta[2]-1.2),1)
  })
})

#' Cox Model with Random Frailty (REML Estimation)
#'
#' Estimates the regression coefficients, random effects, and frailty variance \eqn{\theta} in a Cox proportional hazards model with shared frailty using Restricted Maximum Likelihood (REML).
#'
#' @param data A data frame containing:
#'   - `times`: observed survival or censoring times,
#'   - `status`: event indicator (1 = event, 0 = censored),
#'   - `clusters`: cluster identifiers (e.g., center, group),
#'   - covariates from column 4 onward.
#' @param max_iter Maximum number of Newton-Raphson iterations (default = 300).
#' @param tol Tolerance for convergence (default = 1e-6).
#' @param threshold Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{beta}{Estimated fixed effects (regression coefficients).}
#'   \item{u}{Estimated random effects (frailties) for each cluster.}
#'   \item{theta}{Estimated variance of the random effects.}
#'   \item{p_value}{P-value for testing the null hypothesis \eqn{H_0: \theta = 0} (no frailty).}
#' }
#'
#' @details
#' The function uses a REML-based Newton-Raphson approach to estimate the parameters.
#' A penalized likelihood including a log-likelihood contribution for the frailty variance \eqn{\theta} is maximized.
#'
#' The hypothesis test of \eqn{H_0: \theta = 0} is based on a Wald-type statistic. A high p-value (e.g. > 0.05) may indicate that the random effects (frailties) are negligible.
#'
#' @importFrom stats pchisq
#' @importFrom Matrix Matrix Diagonal
#' @importFrom Matrix sparseMatrix
#'
#' @seealso \code{\link{Ml_Cox}}, \code{\link{logLikelihood_1}}, \code{\link{logLikelihood_2}}
#'
#' @export

Reml_Cox_frailty <- function(data, max_iter = 300, tol = 1e-6, threshold = 1e-5) {
  p <- ncol(data) - 3
  times <- data$times
  status <- data$status
  N <- length(times)

  if (p > 0) {
    X <- Matrix(as.matrix(data[, 4:(p + 3)]), sparse = TRUE)
  } else {
    X <- Matrix(0, N, 0, sparse = TRUE)
  }

  clusters <- data$clusters
  K <- length(unique(clusters))
  if (K==1)
  {stop("There is only one cluster, therefore there are no cluster effect, you might use: \n - method = 'Cox' if you are using 'Parameters_estimation()'\n - 'Ml_Cox()' if you are using 'Reml_Cox_frailty()'")}

  if (p > 0) {
    data_ini <- data.frame(times = times, status = status, clusters = clusters, as.matrix(X))
    gamma_0 <- Ml_Cox(data_ini)$beta
  } else {
    gamma_0 <- numeric(0)
  }

  u_0 <- rep(0, K)
  theta_0 <- 1
  theta_1 <- 2

  Z <- X
  Q <- sparseMatrix(i = 1:N, j = clusters, x = 1, dims = c(N, K))

  if (p > 0) {
    Y <- cbind(Z, Q)
    Y_t <- rbind(Matrix::t(Z), Matrix::t(Q))
  } else {
    Y <- Q
    Y_t <- Matrix::t(Q)
  }

  D <- Matrix(as.numeric(status == 1), ncol = 1, sparse = TRUE)
  eta <- if (p > 0) as.vector(Z %*% gamma_0 + Q %*% u_0) else as.vector(Q %*% u_0)
  W_diag <- exp(eta)
  W <- Diagonal(x = W_diag)

  M <- Matrix(outer(times, times, FUN = function(ti, tj) as.numeric(ti >= tj)), sparse = TRUE)

  Mt <- Matrix::t(M)
  v <- as.vector(Matrix::colSums(M * W_diag))
  a <- numeric(N)
  a[status == 1] <- 1 / v[status == 1]
  A <- Diagonal(x = a)
  b_vec <- as.vector(M %*% a)
  B <- Diagonal(x = b_vec)

  lik_0 <- logLikelihood_1(status, M, W) + logLikelihood_2(u_0, theta_0, K)
  lik_1 <- lik_0 + 1
  iter <- 1

  while ((abs(lik_1 - lik_0) >= tol) && (abs(theta_1 - theta_0) >= tol) && iter <= max_iter) {
    grad <- as.vector(D) - W %*% (M %*% a)
    A2 <- Diagonal(x = a^2)
    negHess <- W %*% (B - M %*% A2 %*% Mt %*% W)

    d <- c(rep(1, p), rep(1 / theta_0, K))
    M_pen <- Diagonal(x = d)

    V <- Y_t %*% negHess %*% Y + M_pen
    rhs_vec <- as.numeric(Y_t %*% grad) - c(rep(0, p), (1 / theta_0) * u_0)
    update_direction <- solve(V, rhs_vec)


    new_par <- c(gamma_0, u_0) + update_direction
    if (p > 0) gamma_0 <- new_par[1:p]
    u_0 <- new_par[(p + 1):(p + K)]

    theta_1 <- theta_0
    invV_diag <- diag(solve(V)[(p + 1):(p + K), (p + 1):(p + K)])
    trace_term <- sum(invV_diag)
    theta_0 <- as.numeric((sum(u_0^2)) / (K - (trace_term / theta_0)))

    if (theta_0 < threshold) {
      stop("The estimated variance of the cluster effect is inferior to the threshold ('threshold' =",threshold,"), we suggest you use the method without frailty:\n - method = 'Cox' if you are using 'Parameters_estimation()'\n - 'Ml_Cox()' if you are using 'Reml_Cox_frailty()'")
    }

    eta <- if (p > 0) as.vector(Z %*% gamma_0 + Q %*% u_0) else as.vector(Q %*% u_0)
    W_diag <- exp(eta)
    W <- Diagonal(x = W_diag)

    v <- as.vector(Matrix::colSums(M * W_diag))
    a <- numeric(N)
    a[status == 1] <- 1 / v[status == 1]
    A <- Diagonal(x = a)
    b_vec <- as.vector(M %*% a)
    B <- Diagonal(x = b_vec)

    lik_0 <- lik_1
    lik_1 <- logLikelihood_1(status, M, W) + logLikelihood_2(u_0, theta_0, K)
    iter <- iter + 1
  }

  V_inv <- solve(V)
  A22 <- V_inv[(p + 1):(p + K), (p + 1):(p + K), drop = FALSE]
  wald_stat <- as.numeric(t(u_0) %*% solve(A22, u_0))
  trace_A22 <- sum(diag(as.matrix(A22)))
  df_eff <- K - trace_A22 / theta_0

  suppressWarnings({
    p_value <- pchisq(wald_stat, df = df_eff, lower.tail = FALSE)
  })
  if (is.nan(p_value)){
    message("The estimated value of theta is too small to allow a reliable statistical test of whether theta = 0, hence the p-value is 'NaN'. \nThis suggests that the cluster effect may be negligible.")
  }

  if (!is.na(p_value) && p_value >= 0.05) {
    message("A p-value greater than 0.05 in the test of theta=0 suggests that the cluster effect may be negligible.\n p-value = ", p_value)
  }

  return(list(beta = gamma_0, u = u_0, theta = theta_0, p_value = p_value))
}

#' Maximum Likelihood Estimation for Cause 1 in Competing Risks
#'
#' Performs maximum likelihood estimation (MLE) for the effect of covariates on the cause-specific hazard function of cause 1 in a competing risks framework.
#'
#' @param data A data frame containing:
#' \describe{
#'   \item{times}{Observed survival or censoring times.}
#'   \item{status}{Event indicator: 0 = censored, 1 = cause 1 (event of interest), 2+ = other causes.}
#'   \item{Covariates}{All columns from the 4th onward are considered as covariates.}
#' }
#' @param max_iter Maximum number of Newton-Raphson iterations (default = 100).
#' @param tol Convergence tolerance for the change in parameter estimates (default = 1e-6).
#'
#' @return A named list with one element:
#' \describe{
#'   \item{beta}{A numeric vector of estimated regression coefficients for the
#'   subdistribution hazard of cause 1. Each coefficient represents the log
#'   effect of the corresponding covariate on the subdistribution hazard.}
#' }
#'
#' @details
#' The function fits a Cox-type model for the subdistribution hazard of cause 1, treating all other causes (including censoring) as censored (i.e., status != 1 becomes 0).
#' It uses a Newton-Raphson procedure to maximize the partial likelihood.
#'
#' The design matrix is reordered internally by event time to facilitate risk set computation.
#'
#' @seealso \code{\link{Reml_Cox_frailty}}, \code{\link{Ml_Cox}}, \code{\link{logLikelihood_1}}
#'
#' @examples
#' \donttest{
#' data <- data.frame(
#'  times = c(0.099, 0.006, 0.260, 0.151, 0.262,
#'            0.019, 0.026, 0.241, 0.017, 0.195,
#'            0.007, 0.057, 0.241, 0.132, 0.044,
#'            0.242, 0.416, 0.096, 0.172, 0.015),
#'  status = c(1, 0, 1, 1, 0,
#'             0, 0, 0, 0, 0,
#'             0, 0, 0, 0, 0,
#'             0, 0, 0, 0, 0),
#'  clusters = c(1,1,1,1,1,
#'               1,1,1,1,1,
#'              2,2,2,2,2,
#'               2,2,2,2,2),
#'  V1 = c(0.720, 0.289, 0.382, 0.995, 0.045,
#'         0.048, 0.042, 0.810, 0.964, 0.781,
#'         0.443, 0.303, 0.902, 0.596, 0.462,
#'         0.521, 0.656, 0.164, 0.261, 0.635)
#')
#'   beta_hat <- Ml_CompRisk(data)
#' }
#'
#' @export
Ml_CompRisk <- function(data, max_iter = 100, tol = 1e-6)
{
  times <- data$times
  status <- data$status
  if (("clusters" %in% names(data)) && ncol(data) == 3){stop("With no covariables, there is nothing to estimate")}
  if (!("clusters" %in% names(data)) && ncol(data) == 2){stop("With no covariables, there is nothing to estimate")}
  if (("clusters" %in% names(data)) && ncol(data) > 3){X <- as.matrix(data[, 4:ncol(data)])}
  if (!("clusters" %in% names(data)) && ncol(data) > 2){X <- as.matrix(data[, 3:ncol(data)])}
  event_of_interest <- (status == 1)
  status_mod <- ifelse(event_of_interest, 1, 0)
  N <- length(times)
  p <- ncol(X)
  beta <- rep(0, p)
  ord <- order(times)
  times_ord <- times[ord]
  status_ord <- status_mod[ord]
  X_ord <- X[ord, , drop = FALSE]
  for (iter in 1:max_iter) {
    eta <- as.vector(X_ord %*% beta)
    exp_eta <- exp(eta)

    U <- rep(0, p)
    I <- matrix(0, p, p)

    for (i in which(status_ord == 1)) {
      R <- which(times_ord >= times_ord[i])

      sum_exp_eta <- sum(exp_eta[R])
      sum_exp_eta_X <- colSums(exp_eta[R] * X_ord[R, , drop = FALSE])
      sum_exp_eta_XX <- t(X_ord[R, , drop = FALSE]) %*% (exp_eta[R] * X_ord[R, , drop = FALSE])

      U <- U + (X_ord[i, ] - sum_exp_eta_X / sum_exp_eta)
      I <- I + (sum_exp_eta_XX / sum_exp_eta) - (sum_exp_eta_X %*% t(sum_exp_eta_X)) / (sum_exp_eta^2)
    }

    step <- solve(I, U)
    beta_new <- beta + step

    if (sqrt(sum(step^2)) < tol) {
      break
    }
    beta <- beta_new
  }

  return(list(beta = beta))
}

#' Create Step Function for Estimated Survival (G)
#'
#' This function creates a right-continuous step function (vectorized) based on
#' estimated values of the censoring survival function at given unique times.
#'
#' The resulting function can be used to evaluate \eqn{G(t)} for any vector of time points.
#'
#' @param unique_times A numeric vector of increasing unique times (e.g., censoring times).
#' @param G_vals A numeric vector of the same length as `unique_times`, corresponding to estimated values of the survival function at each time.
#'
#' @return A function `G(t)` that can be evaluated on a numeric vector of time points.

create_G_function <- function(unique_times, G_vals)
{
  function(t) {
    idx <- findInterval(t, unique_times)
    idx[idx == 0] <- 1  # If t < min(unique_times), return first survival value (usually 1)
    idx[idx > length(G_vals)] <- length(G_vals)  # If t > max(unique_times), return last known value
    return(G_vals[idx])
  }
}

#' REML Estimation for Cause 1 in a Competing Risks Model with Shared Frailty
#'
#' Fits a cause-specific hazard model for cause 1 in a competing risks framework using restricted maximum likelihood (REML) with a shared frailty term for clusters.
#'
#' @param data A data frame with at least the following columns:
#' \describe{
#'   \item{times}{Observed times to event or censoring.}
#'   \item{status}{Event type: 0 = censored, 1 = cause of interest, >1 = other competing risks.}
#'   \item{clusters}{Cluster membership (e.g., centers or subjects).}
#'   \item{covariates}{(Optional) Columns from the 4th onward are used as covariates.}
#' }
#' @param cluster_censoring Logical. If \code{TRUE}, accounts for center-specific censoring using a frailty model on censoring times. Default is \code{FALSE}.
#' @param max_iter Maximum number of iterations for the Newton-Raphson algorithm (default = 300).
#' @param tol Convergence tolerance for the likelihood and frailty variance (default = 1e-6).
#' @param threshold Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).
#'
#' @return A list with:
#' \describe{
#'   \item{beta}{Estimated regression coefficients for the cause 1 hazard.}
#'   \item{u}{Estimated cluster-specific random effects (frailties).}
#'   \item{theta}{Estimated frailty variance.}
#'   \item{p_value}{P-value testing the null hypothesis \eqn{H_0: \theta = 0}.}
#' }
#'
#' @details
#' The function fits a proportional hazards model for cause 1, accounting for unobserved heterogeneity via a shared frailty term on clusters. When \code{cluster_censoring = TRUE}, a frailty-adjusted survival curve is used to correct for informative censoring using either a Kaplan-Meier or Nelson-Aalen-based estimator.
#'
#' If the estimated frailty variance \eqn{\theta} becomes smaller than the provided \code{threshold}, the model reverts to a standard Cox model for cause 1 using \code{\link{Ml_CompRisk}}.
#'
#' @seealso \code{\link{Ml_CompRisk}}, \code{\link{Reml_Cox_frailty}}, \code{\link{logLikelihood_1}}, \code{\link{compute_M_optimized}}
#'
#' @examples
#' \donttest{
#' data <- data.frame(
#'   times = c(4, 6, 10, 12, 3),
#'   status = c(1, 0, 2, 1, 0),
#'   clusters = c(1, 1, 2, 2, 3),
#'   x1 = rnorm(5),
#'   x2 = sample(0:1, 5, replace = TRUE)
#' )
#' result <- Reml_CompRisk_frailty(data)
#' result$beta
#' result$p_value
#' }
#'
#' @importFrom stats pchisq
#' @importFrom Matrix Matrix Diagonal
#' @importFrom Matrix sparseMatrix
#' @export

Reml_CompRisk_frailty <- function(data, cluster_censoring = FALSE, max_iter = 300, tol = 1e-6, threshold = 1e-5) {
  p <- ncol(data) - 3
  times <- data$times
  status <- data$status
  N <- length(times)

  if (p > 0) {
    X <- Matrix(as.matrix(data[, 4:(p + 3)]), sparse = TRUE)
  } else {
    X <- Matrix(0, N, 0, sparse = TRUE)
  }

  clusters <- data$clusters
  K <- length(unique(clusters))
  if (K==1)
    {stop("There is only one cluster, therefore there are no cluster effect, you might use: \n - method = 'CompRisk' if you are using 'Parameters_estimation()'\n - 'Ml_CompRisk()' if you are using 'Reml_CompRisk_frailty()'")}

  if (p > 0) {
    data_ini <- data.frame(times = times, status = status, clusters = clusters, as.matrix(X))
    gamma_0 <- Ml_Cox(data_ini)$beta
  } else {
    gamma_0 <- numeric(0)
  }

  u_0 <- rep(0, K)
  theta_0 <- 1
  theta_1 <- 2

  Z <- X
  Q <- sparseMatrix(i = 1:N, j = clusters, x = 1, dims = c(N, K))

  if (p > 0) {
    Y <- cbind(Z, Q)
    Y_t <- rbind(Matrix::t(Z), Matrix::t(Q))
  } else {
    Y <- Q
    Y_t <- Matrix::t(Q)
  }

  D <- Matrix(as.numeric(status == 1), ncol = 1, sparse = TRUE)
  eta <- if (p > 0) as.vector(Z %*% gamma_0 + Q %*% u_0) else as.vector(Q %*% u_0)
  W_diag <- exp(eta)
  W <- Diagonal(x = W_diag)

  if (cluster_censoring && !(0 %in% status)) {
    cluster_censoring <- FALSE
  }

  if (!cluster_censoring) {
    M <- compute_M_optimized(times, status, "KaplanMeier_Censoring_vectorized", u_bis = rep(0, 1), clusters)
  } else {
    d_cens <- as.integer(status == 0)
    data_bis <- data.frame(times = times, status = d_cens, clusters = clusters, matrix(0, N, 0))
    l <- Reml_Cox_frailty(data_bis)
    u_bis <- l$u
    M <- compute_M_optimized(times, status, "Nelson_Censoring_vectorized", u_bis, clusters)
  }

  Mt <- Matrix::t(M)
  v <- as.vector(Matrix::colSums(M * W_diag))
  a <- numeric(N)
  a[status == 1] <- 1 / v[status == 1]
  A <- Diagonal(x = a)
  b_vec <- as.vector(M %*% a)
  B <- Diagonal(x = b_vec)

  lik_0 <- logLikelihood_1(status, M, W) + logLikelihood_2(u_0, theta_0, K)
  lik_1 <- lik_0 + 1
  iter <- 1

  while ((abs(lik_1 - lik_0) >= tol) && (abs(theta_1 - theta_0) >= tol) && iter <= max_iter) {
    grad <- as.vector(D) - W %*% (M %*% a)
    A2 <- Diagonal(x = a^2)
    negHess <- W %*% (B - M %*% A2 %*% Mt %*% W)

    d <- c(rep(1, p), rep(1 / theta_0, K))
    M_pen <- Diagonal(x = d)

    V <- Y_t %*% negHess %*% Y + M_pen
    rhs_vec <- as.numeric(Y_t %*% grad) - c(rep(0, p), (1 / theta_0) * u_0)
    update_direction <- solve(V, rhs_vec)


    new_par <- c(gamma_0, u_0) + update_direction
    if (p > 0) gamma_0 <- new_par[1:p]
    u_0 <- new_par[(p + 1):(p + K)]

    theta_1 <- theta_0
    invV_diag <- diag(solve(V)[(p + 1):(p + K), (p + 1):(p + K)])
    trace_term <- sum(invV_diag)
    theta_0 <- as.numeric((sum(u_0^2)) / (K - (trace_term / theta_0)))

    if (theta_0 < threshold) {
      stop("The estimated variance of the cluster effect is inferior to the threshold ('threshold' =",threshold,"), we suggest you use the method without frailty:\n - method = 'CompRisk' if you are using 'Parameters_estimation()'\n - 'Ml_CompRisk()' if you are using 'Reml_CompRisk_frailty()'")
    }

    eta <- if (p > 0) as.vector(Z %*% gamma_0 + Q %*% u_0) else as.vector(Q %*% u_0)
    W_diag <- exp(eta)
    W <- Diagonal(x = W_diag)

    v <- as.vector(Matrix::colSums(M * W_diag))
    a <- numeric(N)
    a[status == 1] <- 1 / v[status == 1]
    A <- Diagonal(x = a)
    b_vec <- as.vector(M %*% a)
    B <- Diagonal(x = b_vec)

    lik_0 <- lik_1
    lik_1 <- logLikelihood_1(status, M, W) + logLikelihood_2(u_0, theta_0, K)
    iter <- iter + 1
  }

  V_inv <- solve(V)
  A22 <- V_inv[(p + 1):(p + K), (p + 1):(p + K), drop = FALSE]
  wald_stat <- as.numeric(t(u_0) %*% solve(A22, u_0))
  trace_A22 <- sum(diag(as.matrix(A22)))
  df_eff <- K - trace_A22 / theta_0


  suppressWarnings({
      p_value <- pchisq(wald_stat, df = df_eff, lower.tail = FALSE)
  })
  if (is.nan(p_value)){
    message("The estimated value of theta is too small to allow a reliable statistical test of whether theta = 0, hence the p-value is 'NaN'. \nThis suggests that the cluster effect may be negligible.")
  }

  if (!is.na(p_value) && p_value >= 0.05) {
    message("A p-value greater than 0.05 in the test of theta=0 suggests that the cluster effect may be negligible.\n p-value = ", p_value)
  }

  return(list(beta = gamma_0, u = u_0, theta = theta_0, p_value = p_value))
}

test_that("Reml_CompRisk_frailty works in a typical case.", {
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  res = Reml_CompRisk_frailty(df)
  expect_equal(length(res$beta),n_cov)
  expect_equal(length(res$theta),1)
  expect_equal(length(res$u),n_cluster)
  expect_lte(abs(res$beta[1]-1),1)
  expect_lte(abs(res$beta[2]-1.2),1)
  expect_lte(abs(res$theta - 0.3),1)
  })
})

test_that("Reml_CompRisk_frailty when the variance of the frailty effect is close to 0.",{
  set.seed(123)
  suppressMessages({

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(0,-0.7),theta = 0,cens = TRUE)
  res = Reml_CompRisk_frailty(df)
  expect_equal(length(res$beta),n_cov)
  expect_equal(length(res$theta),1)
  expect_equal(length(res$u),n_cluster)
  expect_lte(abs(res$beta[1]),1)
  expect_lte(abs(res$beta[2]+0.7),1)
  expect_lte(abs(res$theta),1)
  })
})

test_that("Reml_CompRisk_frailty works with a clustering of the censoring times.",{
  set.seed(123)
  suppressMessages({

  n_cov = 4
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2,0,-0.4),theta = 0.01,cens = TRUE)
  res = Reml_CompRisk_frailty(df, cluster_censoring = T)
  expect_equal(length(res$beta),n_cov)
  expect_equal(length(res$theta),1)
  expect_equal(length(res$u),n_cluster)
  expect_lte(abs(res$beta[1]-1),1)
  expect_lte(abs(res$beta[2]-1.2),1)
  expect_lte(abs(res$beta[3]),1)
  expect_lte(abs(res$beta[4]+0.4),1)
  expect_lte(abs(res$theta - 0.01),1)
  })
})

test_that("Reml_CompRisk_frailty works without covariables.",{
  set.seed(123)
  suppressMessages({

  n_cov = 0
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(),theta = 0.3,cens = TRUE)
  res = Reml_CompRisk_frailty(df)
  expect_equal(length(res$beta),n_cov)
  expect_equal(length(res$u),n_cluster)
  expect_equal(length(res$theta),1)
  expect_lte(abs(res$theta - 0.3),1)
  })
})

test_that("The p_value for the test theta = 0 might be computationnaly unavailable and therefore equals 'NaN'.",{
  set.seed(1)

  n_cov = 5
  n_per_cluster = 20
  n_cluster = 15
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2,0,0,1),theta = 0,cens = TRUE)
  expect_message(Reml_CompRisk_frailty(df),"The estimated value of theta is too small to allow a reliable statistical test of whether theta = 0, hence the p-value is 'NaN'. \\nThis suggests that the cluster effect may be negligible.")
})

test_that("When the estimated variance for the frailty effect is too small, it suggests to use a Fine and Gray model.",{
  set.seed(2)

  n_cov = 5
  n_per_cluster = 20
  n_cluster = 15
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2,0,0,1),theta = 0,cens = TRUE)
  expect_error(Reml_CompRisk_frailty(df, threshold = 1e-5),"The estimated variance of the cluster effect is inferior to the threshold \\('threshold' =1e-05\\), we suggest you use the method without frailty:\\n - method = 'CompRisk' if you are using 'Parameters_estimation\\(\\)'\\n - 'Ml_CompRisk\\(\\)' if you are using 'Reml_CompRisk_frailty\\(\\)'")
})

test_that("When there is only one cluster, it suggests to use a Fine and Gray model.",{
  set.seed(123)

  n_cov = 2
  n_per_cluster = 20
  n_cluster = 1
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(0,-0.7),theta = 0,cens = TRUE)
  expect_error(Reml_CompRisk_frailty(df),"There is only one cluster, therefore there are no cluster effect, you might use: \\n - method = 'CompRisk' if you are using 'Parameters_estimation\\(\\)'\\n - 'Ml_CompRisk\\(\\)' if you are using 'Reml_CompRisk_frailty\\(\\)'")
})

#' Maximum Likelihood Estimation for the Cox Model (Optimized)
#'
#' This function computes the maximum likelihood estimates of the regression coefficients in the Cox proportional hazards model using Newton-Raphson iterations.
#'
#' @param data A data frame with columns: `times`, `status`, and covariates starting from the 4th column.
#' @param max_iter Maximum number of iterations (default = 100).
#' @param tol Convergence tolerance for the Euclidean norm (default = 1e-6).
#' @return A named list with one element:
#' \describe{
#'   \item{beta}{A numeric vector of estimated regression coefficients in the
#'   Cox proportional hazards model. Each coefficient represents the log hazard
#'   ratio associated with the corresponding covariate.}
#' }
#' @export
Ml_Cox <- function(data, max_iter = 100, tol = 1e-6) {
  times <- data$times
  status <- data$status
  if (("clusters" %in% names(data)) && ncol(data) == 3){stop("With no covariables, there is nothing to estimate")}
  if (!("clusters" %in% names(data)) && ncol(data) == 2){stop("With no covariables, there is nothing to estimate")}
  if (("clusters" %in% names(data)) && ncol(data) > 3){X <- as.matrix(data[, 4:ncol(data)])}
  if (!("clusters" %in% names(data)) && ncol(data) > 2){X <- as.matrix(data[, 3:ncol(data)])}
  N <- length(times)
  p <- ncol(X)

  beta <- rep(0, p)

  # Tri par temps croissants
  ord <- order(times)
  times <- times[ord]
  status <- status[ord]
  X <- X[ord, , drop = FALSE]

  for (iter in seq_len(max_iter)) {
    eta <- drop(X %*% beta)
    exp_eta <- exp(eta)

    rev_exp_eta <- rev(cumsum(rev(exp_eta)))
    rev_exp_eta_X <- apply(X * exp_eta, 2, function(col) rev(cumsum(rev(col))))

    # Calcule vectorisé de S2
    X_eta <- X * sqrt(exp_eta)
    S2_array <- array(0, dim = c(p, p, N))
    for (i in seq_len(p)) {
      for (j in seq_len(p)) {
        S2_array[i, j, ] <- rev(cumsum(rev(X[, i] * X[, j] * exp_eta)))
      }
    }

    I <- matrix(0, p, p)
    U <- numeric(p)

    for (i in which(status == 1)) {
      S0 <- rev_exp_eta[i]
      S1 <- rev_exp_eta_X[i, ]
      S2 <- S2_array[, , i]

      U <- U + (X[i, ] - S1 / S0)
      I <- I + (S2 / S0) - (tcrossprod(S1) / S0^2)
    }

    step <- tryCatch(solve(I, U), error = function(e) rep(0, p))
    beta_new <- beta + step

    if (sqrt(sum(step^2)) < tol) {
      break
    }
    beta <- beta_new
  }

  return(list(beta = beta))
}

test_that("Ml_Cox does not work without covariables.", {

  n_cov = 0
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(),theta = 0.3,cens = TRUE)
  expect_error(Ml_Cox(df),"With no covariables, there is nothing to estimate")
})

test_that("Ml_Cox works in a typical case.",{
  set.seed(123)

  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)
  res = Ml_Cox(df)
  expect_equal(length(res$beta),n_cov)
  expect_lte(abs(res$beta[1]-1),1)
  expect_lte(abs(res$beta[2]-1.2),1)
})

#' Kaplan-Meier Estimator for Censoring Distribution (Vectorized)
#'
#' Computes the Kaplan-Meier estimator of the censoring distribution at given time points.
#'
#' @param times A numeric vector of observed times.
#' @param status A numeric vector of status indicators (0 = censored, >0 = event).
#' @param unique_times A numeric vector of time points at which to evaluate the estimator.
#'
#' @return A numeric vector of estimated survival probabilities at each unique time.

KaplanMeier_Censoring_vectorized <- function(times, status, unique_times) {

  n <- length(unique_times)
  surv <- numeric(n)
  surv[1] <- 1

  for (i in seq_len(n)) {
    t <- unique_times[i]
    d <- sum(times == t & status == 0)  # number of censored at time t
    R <- sum(times >= t)                # number at risk at time t
    if (R > 0) {
      surv[i] <- ifelse(i == 1, 1, surv[i - 1] * (1 - d / R))
    } else {
      surv[i] <- ifelse(i == 1, 1, surv[i - 1])
    }
  }

  return(surv)
}

#' Log-Likelihood Contribution for Cause 1
#'
#' Computes the contribution to the log-likelihood from individuals who experienced event type 1,
#' using a weighting matrix `W` and an IPCW matrix `M`.
#'
#' @param status A numeric vector indicating the event type:
#' `0` for censored, `1` for cause 1, `2` for other causes.
#' @param M A numeric matrix (usually computed with \code{\link{compute_M_optimized}}), representing
#' IPCW weights between individuals.
#' @param W A sparse matrix (e.g., of class \code{dgCMatrix} from the \pkg{Matrix} package)
#' representing estimated weights or hazards.
#'
#' @return A numeric scalar: the log-likelihood contribution from all individuals who failed from cause 1.
#'
#' @details
#' For each individual `i` such that `status[i] == 1`, the function computes:
#' \deqn{
#' \log W_{ii} - \log\left(\sum_j M_{ij} W_{ij}\right)
#' }
#' and sums this across all such individuals.
#'
#' @seealso [compute_M_optimized()], [Matrix::Matrix-class]


logLikelihood_1 = function(status, M, W)
{
  loglik <- 0
  for (i in which(status == 1)) {
    v <- log(sum(M[i, ] * W@x))
    loglik = loglik + log(W[i, i]) - v
  }
  return(loglik)
}

#' Compute Log-Likelihood Penalty for Random Effects
#'
#' This function computes the second component of the log-likelihood (penalty term)
#' associated with the random effects (frailty terms) in a mixed-effects model.
#'
#' The penalty is derived from the Gaussian assumption on the random effects and
#' involves both the dimension of the random effects vector and the variance parameter \code{theta}.
#'
#' @param u A numeric vector of length \code{K} containing the random effects (frailties).
#' @param theta A positive numeric scalar, representing the variance component of the random effects.
#' @param K An integer representing the number of clusters (i.e., the length of \code{u}).
#'
#' @return A numeric scalar representing the log-likelihood penalty contribution from the random effects.

logLikelihood_2 = function(u, theta, K) {
  penalty <- -(1/2) * (K * log(2 * pi * theta) + (1 / theta) * (t(u) %*% u))
  return(penalty)
}

#' Compute the Weight Matrix M (Optimized)
#'
#' This function computes the matrix of inverse probability of censoring weights (IPCW),
#' used in multicenter competing risks models with frailty. It supports both Kaplan-Meier and Nelson-Aalen estimators
#' for the censoring distribution.
#'
#' @param times A numeric vector of observed times (events or censoring).
#' @param status A numeric vector of status indicators (0 = censored, 1 or 2 = cause of failure).
#' @param f A string, either `"KaplanMeier_Censoring_vectorized"` or `"Nelson_Censoring_vectorized"`,
#' specifying the method to estimate the censoring distribution.
#' @param u_bis A numeric vector of cluster-level random effects (used only if `f == "Nelson_Censoring_vectorized"`).
#' @param clusters A vector indicating cluster membership for each subject (same length as `times`).
#'
#' @return A square matrix `M` of size N x N, where `M[i, j]` is the IPCW weight between subject i and subject j.
#'
#' @details
#' - If `f == "KaplanMeier_Censoring_vectorized"`, then `u_bis` is ignored.
#' - The weight matrix accounts for pairwise contributions, depending on event times and causes.
#'
#' @seealso [KaplanMeier_Censoring_vectorized()], [Nelson_Censoring_vectorized()], [create_G_function()]


compute_M_optimized <- function(times, status, f, u_bis, clusters)
{
  N <- length(times)
  unique_times <- sort(unique(times))

  # Censoring survival estimation
  if (f == "KaplanMeier_Censoring_vectorized") {
    G_vals <- KaplanMeier_Censoring_vectorized(times, status, unique_times)
  }
  if (f == "Nelson_Censoring_vectorized") {
    G_vals <- Nelson_Censoring_vectorized(times, unique_times, status, clusters, u_bis)
  }
  G_func <- create_G_function(unique_times, G_vals)

  # Matrix form for efficient vectorized comparison
  T_mat_i <- matrix(rep(times, each = N), nrow = N)
  T_mat_j <- matrix(rep(times, times = N), nrow = N)
  fail_mat_j <- matrix(rep(status, times = N), nrow = N)

  cond1 <- (T_mat_j <= T_mat_i) & (fail_mat_j > 1)
  cond2 <- (T_mat_j >= T_mat_i)
  cond3 <- (T_mat_j < T_mat_i) & (fail_mat_j <= 1)

  G_T_i <- G_func(times)
  G_T_j <- G_func(times)
  G_T_i_mat <- matrix(rep(G_T_i, each = N), nrow = N)
  G_T_j_mat <- matrix(rep(G_T_j, times = N), nrow = N)

  M <- Matrix(0, nrow = N, ncol = N)
  M[cond1] <- G_T_i_mat[cond1] / G_T_j_mat[cond1]
  M[cond2] <- 1
  M[cond3] <- 0

  return(M)
}

#' Nelson-Aalen Estimator of Censoring Distribution (Vectorized)
#'
#' Computes the estimated censoring survival probabilities using a Nelson-Aalen-type estimator,
#' accounting for shared frailty through a cluster-specific random effect.
#'
#' @param times A numeric vector of observed times.
#' @param unique_times A numeric vector of distinct, sorted time points.
#' @param status A numeric vector where 0 indicates censoring.
#' @param clusters A vector indicating the cluster/group for each observation.
#' @param u_bis A numeric vector of cluster-specific random effects (e.g., frailty values).
#'
#' @return A numeric vector of estimated censoring survival probabilities for each individual.


Nelson_Censoring_vectorized <- function(times, unique_times, status, clusters, u_bis) {
  check_data_format(data.frame(times = times, status = status, clusters = clusters))

  n_times <- length(unique_times)
  N <- length(times)

  # Step 1: Estimate cumulative hazard function H(t) at each unique time
  d_vec <- sapply(unique_times, function(t) sum(times == t & status == 0))  # censored at t
  R_vec <- sapply(unique_times, function(t) {
    ind <- which(times >= t)
    sum(exp(u_bis[clusters[ind]]))
  })

  lambda_hat <- ifelse(R_vec > 0, d_vec / R_vec, 0)
  H_vec <- cumsum(lambda_hat)  # Estimated Nelson-Aalen cumulative hazard

  # Step 2: Map each individual's time to H(t_i)
  H_ti <- H_vec[findInterval(times, unique_times)]

  # Step 3: Compute censoring survival probability for each individual
  S_C <- exp(-H_ti * exp(u_bis[clusters]))

  return(S_C)
}

#' Check the Format of a Data Frame for Multicentre Competing Risks with Frailty
#'
#' Validates whether the submitted data frame is correctly structured for the analysis
#' of multicentre competing risks with frailty.
#'
#' @param df A data frame to be checked.
#'
#' @return `TRUE` if `df` has the expected structure.
#' Otherwise, the function stops with an error describing the formatting issue.
#'
#' @examples
#' \donttest{
#' n_cov = 1
#' n_cluster = 5
#' n_per_cluster = 100
#' n = n_cluster * n_per_cluster
#' a1 = 0
#' b1 = 1
#' G <- rep(1:n_cluster, each = n_per_cluster)
#' Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
#' df = simulate_data(
#'   G,
#'   Z,
#'   prop = 0.6,
#'   beta = c(0),
#'   theta = 0.01,
#'   cens = TRUE,
#'   pcens = 0.25,
#'   tau = 0
#' )
#' check_data_format(df)
#' }
#'
#' @export


check_data_format <- function(df)
{
  if (!is.data.frame(df)) {
    stop("The argument must be a data frame.")
  }

  n_cols <- ncol(df)
  if (n_cols < 3) {
    stop(
      "The data frame must contain at least 3 columns:\n - 'times': the observed time,\n - 'status': 0 for right-censoring, i in [1,n] otherwise, where i is the cause of failure,\n - 'clusters': cluster/group indicator."
    )
  }
  n_row <- nrow(df)
  if (n_row == 0) {
    stop(
      "The dataframe must contain at least one row, otherwise, these is nothing to analyse."
    )
  }

  col_names <- colnames(df)
  expected_names <- c("times", "status", "clusters")
  for (i in 1:3) {
    if (col_names[i] != expected_names[i]) {
      stop(paste0("Column ", i, " must be named '", expected_names[i], "'."))
    }
  }

  if (!(is.numeric(df$times) && all(is.finite(df$times)) && all(df$times >= 0)))
    stop("Some values in 'times' are missing (NA) or non-positive numeric.")
  if (!(is.numeric(df$status) && all(is.finite(df$status)) && all(df$status == floor(df$status)) && all(df$status >= 0)))
    stop("Some values in 'status' are missing (NA) or non-positive integer.")
  if (!(is.numeric(df$clusters) && all(is.finite(df$clusters)) && all(df$clusters == floor(df$clusters)) && all(df$clusters >= 0)))
    stop("Some values in 'clusters' are missing (NA) or non-positive integer.")
  if (n_cols > 3) {
    covariate_cols <- df[, 4:n_cols, drop = FALSE]
    if (!all(sapply(covariate_cols, is.numeric)) || any(is.na(covariate_cols))) {
      stop("All covariates (columns 4 to end) must be numeric and contain no missing values (NA).")
    }
  }
  return(TRUE)
}

test_that("check_data_format works", {

  expect_error(check_data_format(1),"The argument must be a data frame.")
})

test_that("the data_frame cannot be empty.",{
  m = matrix(0, nrow = 0, ncol = 4)
  expect_error(check_data_format(data.frame(m)),"The dataframe must contain at least one row, otherwise, these is nothing to analyse.")
})

test_that("The data.frame require at least 3 columns.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_2 = data.frame(times = df_1$times,status = df_1$status)
  expect_error(check_data_format(df_2),"The data frame must contain at least 3 columns:\\n - 'times': the observed time,\\n - 'status': 0 for right-censoring, i in \\[1,n\\] otherwise, where i is the cause of failure,\\n - 'clusters': cluster/group indicator.")
})

test_that("The first column has to be named 'times'.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_3 = data.frame(status = df_1$status,clusters = df_1$clusters,as.matrix(df_1[,4:length(df_1[1,])]))
  expect_error(check_data_format(df_3),"Column 1 must be named 'times'.")
})

test_that("The second column has to be named 'status'.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_4 = data.frame(times = df_1$times,clusters = df_1$clusters,as.matrix(df_1[,4:length(df_1[1,])]))
  expect_error(check_data_format(df_4),"Column 2 must be named 'status'.")
})

test_that("The third column has to be named 'clusters'.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_5 = data.frame(times = df_1$times,status = df_1$status,as.matrix(df_1[,4:length(df_1[1,])]))
  expect_error(check_data_format(df_5),"Column 3 must be named 'clusters'.")
})

test_that("'times' is a vector of positive numeric values.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_6 = df_1
  df_6$times[1] <- NA
  expect_error(check_data_format(df_6), "Some values in 'times' are missing \\(NA\\) or non-positive numeric.")
})

test_that("'status' is a vector of positive integer values.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_7 = df_1
  df_7$status[1] <- NA
  expect_error(check_data_format(df_7), "Some values in 'status' are missing \\(NA\\) or non-positive integer.")
})

test_that("'clusters' is a vector of positive integer values.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_8 = df_1
  df_8$clusters[1] <- NA
  expect_error(check_data_format(df_8), "Some values in 'clusters' are missing \\(NA\\) or non-positive integer.")
})

test_that("Covariables are numeric.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df_1 = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.3,cens = TRUE)

  df_9 = df_1
  df_9[1,length(df_9)] <- NA
  expect_error(check_data_format(df_9), "All covariates \\(columns 4 to end\\) must be numeric and contain no missing values \\(NA\\).")
})

test_that("simulate_data works with censoring.", {
  n_cov = 0
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(),theta = 0.3,cens = TRUE)
  expect_equal(check_data_format(df), T)
})

test_that("simulate_data works without censoring.", {
  n_cov = 0
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(),theta = 0.3,cens = FALSE)
  expect_equal(check_data_format(df), T)
})

test_that("simulate_data works when the variance of the frailty effect is equal to 0.", {
  n_cov = 0
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  df = simulate_data(G,Z,prop = 0.6,beta = c(),theta = 0,cens = TRUE)
  expect_equal(check_data_format(df), T)
})

test_that("When beta is not defined, simulate_data does not work.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  expect_error(simulate_data(G,Z,prop = 0.6,beta = NULL,theta = 0.3,cens = FALSE), "You must provide beta when Z is given.")
})

test_that("When the length of beta and the length of the covariable of an individual are not the same, simulate_data does not work.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:n_cluster, each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  expect_error(simulate_data(G,Z,prop = 0.6,beta = c(1),theta = 0.3,cens = FALSE), "Length of beta must match the number of columns in Z.")
})

test_that("When the sample size does not match the number of cluster observations, simulate_data does not work.",{
  n_cov = 2
  n_per_cluster = 15
  n_cluster = 20
  n = n_cluster * n_per_cluster
  G = rep(1:(n_cluster-1), each = n_per_cluster)
  Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)
  expect_error(simulate_data(G,Z,prop = 0.6,beta = c(1,0),theta = 0.3,cens = FALSE), "The sample size does not match the number of cluster observations.")
})

#' Simulate clustered competing risks data with shared frailty
#'
#' This function simulates clustered time-to-event data under a competing risks framework
#' with shared frailty and the possibility of random censoring. Cause 1 is modeled through
#' an inversion method of the subdistribution function, and cause 2 is introduced when inversion fails.
#'
#' @param G A vector of group or cluster identifiers (length \code{N}). Each value indicates which cluster the individual belongs to.
#' @param Z A matrix of covariates (dimensions \code{N x p}). Can be set to \code{NULL} if no covariates are used.
#' @param prop Proportion of individuals susceptible to cause 1 (default: \code{0.6}).
#' @param beta A numeric vector of regression coefficients, one per covariate (length \code{p}).
#' @param theta Variance of the shared frailty term for event times (cause 1).
#' @param cens Logical, indicating whether censoring should be simulated (default: \code{FALSE}).
#' @param pcens Target censoring proportion (default: \code{0.25}).
#' @param tau Variance of the shared frailty term for censoring times (default: \code{0}).
#'
#' @return A `data.frame` with one row per individual and at least the columns:
#' \describe{
#'   \item{times}{Observed follow-up time.}
#'   \item{status}{Event indicator: `0` for censoring, `1` for cause 1,
#'   `2` for cause 2.}
#'   \item{clusters}{Cluster identifier.}
#' }
#' If covariates are simulated, additional numeric covariate columns are included.
#'
#' @examples
#' \donttest{
#' n_cov <- 1
#' n_cluster <- 5
#' n_per_cluster <- 100
#' n <- n_cluster * n_per_cluster
#'
#' G <- rep(1:n_cluster, each = n_per_cluster)
#' Z <- matrix(runif(n * n_cov), ncol = n_cov)
#'
#' df <- simulate_data(
#'   G = G,
#'   Z = Z,
#'   prop = 0.6,
#'   beta = c(0.5),
#'   theta = 0.01,
#'   cens = TRUE,
#'   pcens = 0.25,
#'   tau = 0.01
#' )
#'
#' head(df)
#' table(df$status)
#' }
#'
#' @importFrom stats rexp rnorm runif uniroot
#' @export


simulate_data <- function(G, Z = NULL, prop = 0.6, beta = NULL, theta = 0.5, cens = FALSE, pcens = 0.25, tau = 0.5) {
  N <- length(G)
  k <- length(unique(G))

  p <- if (!is.null(Z)) if (is.null(dim(Z))) 1 else ncol(Z) else 0

  if (p > 0 && is.null(beta)) stop("You must provide beta when Z is given.")
  if (p > 0 && length(beta) != p) stop("Length of beta must match the number of columns in Z.")
  if (p > 0 && length(G) != nrow(Z)){stop("The sample size does not match the number of cluster observations.")}

  # Frailty
  cluster_id <- as.numeric(factor(G))
  frailty <- rnorm(k, mean = 0, sd = sqrt(theta))[cluster_id]

  # Predictors
  linpred <- if (p > 0) as.vector(Z %*% beta) + frailty else frailty

  # Simulate uniform random variable
  u <- runif(N)

  # Inverse CDF of cause 1 failure time
  tf <- -log(1 - u) / (prop * exp(linpred))  # Exponential approximation

  # Censure
  if (cens) {
    frailty_cens <- rnorm(k, mean = 0, sd = sqrt(tau))[cluster_id]

    # Calibrate censoring distribution
    # Assume base rate such that proportion censored ~ pcens
    base_rate <- uniroot(function(r) {
      mean(rexp(N, rate = r * exp(frailty_cens)) > tf) - pcens
    }, interval = c(0.001, 100))$root

    censure_times <- rexp(N, rate = base_rate * exp(frailty_cens))
  } else {
    censure_times <- rep(max(tf) + 1, N)
  }

  # Observed data
  X <- pmin(tf, censure_times)
  status <- as.integer(tf <= censure_times)

  # Build output
  df <- data.frame(
    times = X,
    status = status,
    clusters = G
  )
  if (p > 0) df <- cbind(df, as.data.frame(Z))

  return(df)
}

---
title: "Tests Kats"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tests Kats}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(FrailtyCompRisk)
library(tibble)
library(dplyr)
library(knitr)
library(kableExtra)
```

Multi_tests with simulate_data()


fonctions annexes

```{r}
compute_mean_mse <- function(df, true_values) {
  if (!is.data.frame(df)) stop("L'entrée doit être un data.frame.")
  if (!is.numeric(true_values)) stop("true_values doit être un vecteur numérique.")
  if (ncol(df) != length(true_values)) {
    stop("Le nombre de colonnes du data.frame doit correspondre à la longueur de true_values.")
  }

  means <- colMeans(df, na.rm = TRUE)
  
  true_mat <- matrix(rep(true_values, each = nrow(df)), nrow = nrow(df), byrow = FALSE)
  mse <- colMeans((df - true_mat)^2, na.rm = TRUE)
  
  result <- data.frame(
    variable = colnames(df),
    true_value = true_values,
    mean = means,
    mse = mse
  )
  
  return(result)
}
```


Ml_Cox()

```{r}
Multi_tests = function(N_test,
                       n_cov,
                       n_cluster,
                       n_per_cluster,
                       a1,
                       b1)
{
  n = n_cluster * n_per_cluster
  results <- data.frame(matrix(NA, nrow = N_test, ncol = (n_cov)))
  if (n_cov > 0) {
    colnames(results) <- paste0("beta", 1:n_cov)
  } else{
    stop("Covariables missing")
  }
  for (i in 1:N_test)
  {
    G <- rep(1:n_cluster, each = n_per_cluster)
    if (n_cov == 0) {
      Z <- matrix(0, n, 0)
    } else {
      Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
    }
    df = simulate_data(
      G,
      Z,
      prop = 0.6,
      beta = c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35),
      theta = 0,
      cens = TRUE,
      pcens = 0.25,
      tau = 0.5
    )
    df = data.frame(times = df$times, status = df$status,clusters = rep(1,length(G)),as.matrix(df[,4:(n_cov + 3)]))
    ind <- which(df$status > 1)
    df$status[ind] <- 1
    res <- tryCatch(
      Ml_Cox(df),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)
      }
    )
    
    if (!is.null(res)) {
      if (n_cov > 0) {
        results[i, 1:n_cov] <- res$beta
      }
    }
  }
  return(results)
}

M = Multi_tests(N_test = 100,n_cov = 8,n_cluster = 20,n_per_cluster = 25,a1 = 0,b1=1)
m = compute_mean_mse(M, c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35))
m

```


Reml_Cox_frailty()

```{r}
Multi_tests = function(N_test,
                       n_cov,
                       n_cluster,
                       n_per_cluster,
                       a1,
                       b1)
{
  suppressMessages({
  n = n_cluster * n_per_cluster
  results <- data.frame(matrix(NA, nrow = N_test, ncol = (n_cov + 1)))
  if (n_cov > 0) {
    colnames(results) <- c(paste0("beta", 1:n_cov), "theta_hat")
  } else{
    colnames(results) <- "theta_hat"
  }
  for (i in 1:N_test)
  {
    G <- rep(1:n_cluster, each = n_per_cluster)
    if (n_cov == 0) {
      Z <- matrix(0, n, 0)
    } else {
      Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
    }
    df = simulate_data(
      G,
      Z,
      prop = 0.6,
      beta = c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35),
      theta = 0,
      cens = TRUE,
      pcens = 0.25,
      tau = 0.5
    )
    check_data_format(df)
    res <- tryCatch(
      Reml_Cox_frailty(df, max_iter = 100),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)
      }
    )
   if (is.null(res)) {
     res_bis <- tryCatch(
      Ml_Cox(df, max_iter = 100),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)})
     res <- list(beta = res_bis$beta,theta = 0)
   }
    if (!is.null(res)) {
      if (n_cov > 0) {
        results[i, 1:n_cov] <- res$beta
        results[i, "theta_hat"] <- res$theta
      } else{
        results[i, "theta_hat"] <- res$theta
      }
    }
  }
  })
  return(results)
}

M = Multi_tests(N_test = 100,n_cov = 8,n_cluster = 15, n_per_cluster = 20,a1 = 0,b1 = 1)
m = compute_mean_mse(M,c(0,0.25,0.12,0.33,0.24,1,-0.2,-0.35,0))
m
```


Ml_CompRisk()

```{r}
Multi_tests = function(N_test,
                       n_cov,
                       n_cluster,
                       n_per_cluster,
                       a1,
                       b1)
{
  n = n_cluster * n_per_cluster
  results <- data.frame(matrix(NA, nrow = N_test, ncol = (n_cov)))
  if (n_cov > 0) {
    colnames(results) <- paste0("beta", 1:n_cov)
  } else{
    stop("Covariables missing")
  }
  for (i in 1:N_test)
  {
    G <- rep(1:n_cluster, each = n_per_cluster)
    if (n_cov == 0) {
      Z <- matrix(0, n, 0)
    } else {
      Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
    }
    df = simulate_data(
      G,
      Z,
      prop = 0.6,
      beta = c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35),
      theta = 0,
      cens = TRUE,
      pcens = 0.25,
      tau = 0.5
    )
    df = data.frame(times = df$times, status = df$status, clusters = rep(1,length(G)), as.matrix(df[,4:(n_cov + 3)]))
    res <- tryCatch(
      Ml_CompRisk(df),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)
      }
    )
    
    if (!is.null(res)) {
      if (n_cov > 0) {
        results[i, 1:n_cov] <- res$beta
      }
    }
  }
  return(results)
}

M = Multi_tests(N_test = 100,n_cov = 8,n_cluster = 20,n_per_cluster = 25,a1 = 0,b1 = 1)
m = compute_mean_mse(M, c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35))
m
```


Reml_CompRisk_frailty()

```{r}
Multi_tests = function(N_test,
                       n_cov,
                       n_cluster,
                       n_per_cluster,
                       a1,
                       b1)
{
  suppressMessages({
  n = n_cluster * n_per_cluster
  results <- data.frame(matrix(NA, nrow = N_test, ncol = (n_cov + 1)))
  if (n_cov > 0) {
    colnames(results) <- c(paste0("beta", 1:n_cov), "theta_hat")
  } else{
    colnames(results) <- "theta_hat"
  }
  for (i in 1:N_test)
  {
    G <- rep(1:n_cluster, each = n_per_cluster)
    if (n_cov == 0) {
      Z <- matrix(0, n, 0)
    } else {
      Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
    }
    df = simulate_data(
      G,
      Z,
      prop = 0.6,
      beta = c(0, 0.25, 0.12, 0.33, 0.24, 1, -0.2, -0.35),
      theta = 0,
      cens = TRUE,
      pcens = 0.25,
      tau = 0.5
    )
    check_data_format(df)
    res <- tryCatch(
      Reml_CompRisk_frailty(df, max_iter = 100),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)
      }
    )
   if (is.null(res)) {
     res_bis <- tryCatch(
      Ml_CompRisk(df, max_iter = 100),
      error = function(e) {
        message("Erreur à l’itération ", i, " : ", e$message)
        return(NULL)})
     res <- list(beta = res_bis$beta,theta = 0)
   }
    if (!is.null(res)) {
      if (n_cov > 0) {
        results[i, 1:n_cov] <- res$beta
        results[i, "theta_hat"] <- res$theta
      } else{
        results[i, "theta_hat"] <- res$theta
      }
    }
  }
  })
  return(results)
}

M = Multi_tests(N_test = 100,n_cov = 8,n_cluster = 15, n_per_cluster = 20,a1 = 0,b1 = 1)
m = compute_mean_mse(M,c(0,0.25,0.12,0.33,0.24,1,-0.2,-0.35,0))
m
```



Creation of huge data sets with numerous variables

```{r}
generate_full_table <- function(
  beta_vec,
  theta_vec,
  N_vec,
  K_vec,
  n_sim = 100,
  pcens = 0.25
) {
  iter <- 1
  n_iter <- n_sim * length(N_vec) * length(K_vec) * length(theta_vec) * length(beta_vec)
  full_results <- list()
  idx <- 1

  suppressWarnings(suppressMessages({
    for (theta in theta_vec) {
      for (beta in beta_vec) {
        beta_clean <- as.numeric(beta)
        n_cov <- length(beta_clean)

        for (N in N_vec) {
          for (K in K_vec) {
            n_per_cluster <- N / K
            config_id <- paste0("gamma=", paste(beta_clean, collapse = ","),
                                "  theta=", theta,
                                "  N=", N,
                                "  K=", K)
            cat("Starting :", config_id, "\n")

            results_1 <- matrix(NA, nrow = n_sim, ncol = n_cov)
            results_2 <- rep(NA, n_sim)

            for (i in 1:n_sim) {
              cat("iteration", iter, "out of", n_iter, "\n")
              G <- rep(1:K, each = n_per_cluster)
              Z <- if (n_cov == 0) matrix(0, N, 0) else matrix(runif(N * n_cov, 0, 1), ncol = n_cov)

              df <- simulate_data(G, Z, prop = 0.6, beta = beta_clean, theta = theta,
                                  cens = TRUE, pcens = pcens, tau = 0.5)
              check_data_format(df)
              res <- tryCatch(
                Reml_CompRisk_frailty(df),
                error = function(e) return(NULL)
              )
              if (!is.null(res)) {
                if (n_cov > 0) {
                  results_1[i, ] <- res$beta
                }
                results_2[i] <- res$theta
              }
              iter <- iter + 1
            }
            mean_gamma_hat <- c()
            mse_gamma_hat <- c()
            gamma_vals <- c()

            if (n_cov > 0) {
              mean_gamma_hat <- colMeans(results_1, na.rm = TRUE)
              mse_gamma_hat <- colMeans((results_1 - matrix(rep(beta, each = n_sim), nrow = n_sim))^2, na.rm = TRUE)
            }
            
            mean_theta_hat <- mean(results_2, na.rm = TRUE)
            mse_theta_hat <- mean((results_2 - rep(theta, n_sim))^2, na.rm = TRUE)
            
            gamma_data <- tibble(
              gamma = list(beta),
              mean_gamma_hat = list(mean_gamma_hat),
              mse_gamma_hat = list(mse_gamma_hat)
            )
            
            full_results[[idx]] <- bind_cols(
              tibble(
                N = N,
                K = K,
                theta = theta,
                mean_theta_hat = mean_theta_hat,
                mse_theta_hat = mse_theta_hat
              ),
              gamma_data
            )
            idx <- idx + 1
            cat("Finished :", config_id, "\n")
          }
        }
      }
    }
  }))

  final_df <- do.call(rbind, full_results)
  return(list(final_df = final_df,beta_vec = beta_vec))
}

#big_table <- generate_full_table(beta_vec = list(a=c(0),b=c(0.375)), theta_vec = c(0, 0.1, 0.6), N_vec = c(200, 500), K_vec = c(5, 20), n_sim = 100, pcens = 0.25)

#big_table$final_df
```

```{r}
export_to_latex <- function(final_df, file = "simulation_results.tex") {
  # On applique une ligne par simulation avec une cellule contenant les vecteurs formatés
  formatted_df <- final_df %>%
    mutate(
      gamma = sapply(gamma, function(x) paste0("(", paste(round(x, 3), collapse = ", "), ")")),
      mean_gamma_hat = sapply(mean_gamma_hat, function(x) paste0("(", paste(round(x, 3), collapse = ", "), ")")),
      mse_gamma_hat = sapply(mse_gamma_hat, function(x) paste0("(", paste(round(x, 5), collapse = ", "), ")")),
      mean_theta_hat = round(mean_theta_hat, 3),
      mse_theta_hat = round(mse_theta_hat, 5)
    )

  # Affichage en LaTeX
  tex_table <- kable(formatted_df, format = "latex", booktabs = TRUE, escape = FALSE,
                     col.names = c("N", "K", "$\\theta$", "$\\widehat{\\theta}$", "MSE($\\theta$)",
                                   "$\\gamma$", "$\\widehat{\\gamma}$", "MSE($\\gamma$)")) %>%
    kable_styling(latex_options = c("striped", "hold_position"), full_width = FALSE)

  # Sauvegarde dans un fichier
  writeLines(tex_table, file)
}
```
```{r}
#export_to_latex(big_table$final_df,"tableau_Sandrine.tex")
```

---
title: "FrailtyCompRisk"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{FrailtyCompRisk}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(FrailtyCompRisk)
```

# Package FrailtyCompRisk

## Overview

The package **FrailtyCompRisk** is originally designed for statisticians
and epidemiologists in order to analyze time-to-event data where
individuals are nested within centers (e.g., hospitals or clinics), and
where multiple causes of failure may occur.

The package is build upon the random-effects model for the
subdistribution hazard presented in "Analysing multicentre competing
risks data with a mixed proportional hazards model for the
subdistribution" [Katsahian S, Resche-Rigon M, Chevret S, Porcher R.].

## The Model:

Consider $N$ observations of a time-to-event variable $T$, a cause of
event $\epsilon$, and a covariate $Z$:
$(T_i, \epsilon_i, Z_i)_{i=1,\dots,N}$. Let $\epsilon = 1$ denote the
cause of failure of interest.

The cumulative incidence function (CIF) for cause 1 is defined as: $$
F_1(t)=P(T \le t,\epsilon = 1)
$$ Fine and Gray proposed a proportional hazards model for the
subdistribution hazard associated with $F_1(t)$, defined as: $$
\alpha_1(t) = \frac{\frac{dF_1(t)}{dt}}{1 - F_1(t)}
$$ The subdistribution hazard for individual $i$ is modeled as:

$$
\alpha_{1}(t | Z_i, u) = \alpha_{1,0}(t) e^{(Z_i^\top \beta)}
$$ where:

-   $\alpha_{1,0}(t)$ is an unspecified baseline subdistribution hazard,

-   $\beta$ is the vector of covariate effects.

This model assumes that all individuals are independent. To account for
clustering (e.g., patients within centers), a random effect is added:

Let $k(i) \in {1, \dots, K}$ be the cluster (center) to which subject
$i$ belongs. Then the subdistribution hazard becomes:

$$
\alpha_{1,k}(t | Z_i, u) = \alpha_{1,0}(t) e^{(Z_i^\top \beta \ + \ u_{k(i)})}
$$ where $u_k \sim \mathcal{N}(0, \theta)$ are independent Gaussian
random effects for each cluster $k$, with variance $\theta$.

## Method

The package uses a restricted maximum likelihood (REML) or maximum
likelihood (ML) approach to estimate the parameters $\beta$, $\theta$,
and $u_k$. The estimation relies on a Newton-Raphson optimization
routine applied to the (restricted) log-likelihood.

## Features

The package provides several main functions, $\\$
`Reml_CompRisk_frailty()` Estimates $\beta$, $\theta$, and $u_k$ under
the subdistribution hazard model with random effects. Also returns the
$p$-value for testing $H_0: \theta = 0$.

`Reml_CompRisk_frailty()` can take 5 arguments:

1.  `data` (mandatory), a data frame with the following columns:

    -   `times` : Observed times to event or censoring.

    -   `status` : Event type: 0 = censored, 1 = cause of interest, \>1
        = other competing risks.

    -   `clusters` : Cluster membership (e.g., centers or subjects).

    -   `covariates` : (optional) Columns from the 4th onward are used
        as covariates.

2.  `cluster_censoring` (optional): Logical. If TRUE, accounts for
    center-specific censoring using a frailty model on censoring times.
    Default is FALSE.

3.  `max_iter` (optional): Maximum number of iterations for the
    Newton-Raphson algorithm (default = 300).

4.  `tol` (optional): Convergence tolerance for the likelihood and
    frailty variance (default = 1e-6).

5.  `threshold` (optional): Lower bound for the frailty variance
    parameter $\theta$. If the estimated value falls below this
    threshold, frailty is considered negligible (default = 1e-5).

$\\$

`Ml_CompRisk()` Estimates $\beta$ for the subdistribution hazard model
without random effects.

`Ml_CompRisk()` can take 3 arguments:

1.  `data` (mandatory), a data frame with the following columns:

    -   `times` : Observed times to event or censoring.

    -   `status` : Event type: 0 = censored, 1 = cause of interest, \>1
        = other competing risks.

    -   `covariates` : (optional) Columns from the 3th onward are used
        as covariates.

2.  `max_iter` (optional): Maximum number of iterations for the
    Newton-Raphson algorithm (default = 100).

3.  `tol` (optional): Convergence tolerance for the likelihood and
    frailty variance (default = 1e-6).

$\\$

`Reml_Cox_frailty()` Estimates $\beta$, $\theta$, and $u_k$ under a Cox
model with random effects, and provides a $p_{value}$ for testing
$\theta = 0$.

`Reml_Cox_frailty()` can take 4 arguments:

1.  `data` (mandatory), a data frame with the following columns:

    -   `times` : Observed times to event or censoring.

    -   `status` : Event type: 0 = censored, 1 = cause of interest

    -   `clusters` : Cluster membership (e.g., centers or subjects).

    -   `covariates` : (optional) Columns from the 4th onward are used
        as covariates.

2.  `max_iter` (optional): Maximum number of iterations for the
    Newton-Raphson algorithm (default = 300).

3.  `tol` (optional): Convergence tolerance for the likelihood and
    frailty variance (default = 1e-6).

4.  `threshold` (optional): Lower bound for the frailty variance
    parameter $\theta$. If the estimated value falls below this
    threshold, frailty is considered negligible (default = 1e-5).

$\\$

`Ml_Cox()` Estimates $\beta$ under a standard Cox proportional hazards
model.

`Ml_Cox()` can take 3 arguments:

1.  `data` (mandatory), a data frame with the following columns:

    -   `times` : Observed times to event or censoring.

    -   `status` : Event type: 0 = censored, 1 = cause of interest

    -   `covariates` : (optional) Columns from the 3th onward are used
        as covariates.

2.  `max_iter` (optional): Maximum number of iterations for the
    Newton-Raphson algorithm (default = 100).

3.  `tol` (optional): Convergence tolerance for the likelihood and
    frailty variance (default = 1e-6).

`Parameters_estimation()` Encapsulates the four previous methods into
one unified function.

`Parameters_estimation()` can take 6 arguments:

1.  `data` (mandatory), a data frame with the following columns:

    -   `times` : Observed times to event or censoring.

    -   `status` : Event type: 0 = censored, 1 = cause of interest, \>1
        = other competing risks.

    -   `clusters` : Cluster membership (e.g., centers or subjects).

    -   `covariates` : (optional) Columns from the 4th onward are used
        as covariates.

2.  `method` (optional):

    -   "CompRisk_frailty" (default): Competing risks model for cause 1
        with shared frailty.
    -   "CompRisk": Competing risks model for cause 1 without frailty.
    -   "Cox_frailty": Cox proportional hazards model with shared
        frailty.
    -   "Cox": Standard Cox proportional hazards model.

3.  `cluster_censoring` (optional): Logical. If TRUE, accounts for
    center-specific censoring using a frailty model on censoring times.
    Default is FALSE.

4.  `max_iter` (optional): Maximum number of iterations for the
    Newton-Raphson algorithm (default = 100 or 300, depends on the
    method chosen).

5.  `tol` (optional): Convergence tolerance for the likelihood and
    frailty variance (default = 1e-6).

6.  `threshold` (optional): Lower bound for the frailty variance
    parameter $\theta$. If the estimated value falls below this
    threshold, frailty is considered negligible (default = 1e-5).

$\\$

`simulate_data()` Simulates clustered time-to-event data with competing
risks.

`simulate_data()` can take 8 arguments:

1.  `G` : a vector of group or cluster identifiers (length N, where N is
    the size sample). Each value indicates which cluster the individual
    belongs to.

2.  `Z` : a matrix of covariates (dimensions N x p), where p is the size
    of the vector describing an individual). Can be set to Matrix(0,0,N)
    if no covariates are used.

3.  `prop` : proportion of individuals susceptible to cause 1 (default:
    0.6).

4.  `beta` : a numeric vector of regression coefficients, one per
    covariate (p).

5.  `theta` : variance of the shared frailty term for event times (cause
    1).

6.  `cens` : logical, indicating whether censoring should be simulated
    (default: FALSE).

7.  `pcens` : target censoring proportion (default: 0.25).

8.  `tau` : variance of the shared frailty term for censoring times
    (default: 0). $\\$

`check_data_format()` Verifies input data structure and formatting.

`check_data_format()` can only take 1 argument, a data frame. $\\$

## Examples

```{r}
set.seed(123)

n_cov = 2
n_per_cluster = 15
n_cluster = 20
n = n_cluster * n_per_cluster

G = rep(1:n_cluster, each = n_per_cluster)
Z = matrix(rnorm(n*n_cov,0,1),ncol = n_cov)

df = simulate_data(G,Z,prop = 0.6,beta = c(1,1.2),theta = 0.6,cens = TRUE)

 #Estimate using REML with frailty

res <- Reml_CompRisk_frailty(df)

 #Print estimated coefficients

res$beta
res$theta
```

## Computational Complexity

The algorithm involves computing weighted matrices and solving penalized
systems in each iteration. For $N$ individuals, $K$ clusters and $p$
covariables describing each individual:

-   Sparse matrix multiplication and integration steps: \~
    $\mathcal{O}(N^2)$

-   Newton-Raphson updates (inversion of Fisher information matrix): \~
    $\mathcal{O}((p + K)^3)$ in the worst case

In total, the complexity of the algorithm is
\~$\mathcal{O}(N^2 + Np + NK + (p + K)^3)$ in the worst case.

However, the implementation uses sparse matrices (`Matrix` package) to
significantly reduce practical computational cost.

## References

The `Matrix` package

Katsahian S, Resche-Rigon M, Chevret S, Porcher R. (2006). *Analysing
multicentre competing risks data with a mixed proportional hazards model
for the subdistribution.* *Statistics in Medicine*, 25(26), 4267–4278.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLikelihood_2.R
\name{logLikelihood_2}
\alias{logLikelihood_2}
\title{Compute Log-Likelihood Penalty for Random Effects}
\usage{
logLikelihood_2(u, theta, K)
}
\arguments{
\item{u}{A numeric vector of length \code{K} containing the random effects (frailties).}

\item{theta}{A positive numeric scalar, representing the variance component of the random effects.}

\item{K}{An integer representing the number of clusters (i.e., the length of \code{u}).}
}
\value{
A numeric scalar representing the log-likelihood penalty contribution from the random effects.
}
\description{
This function computes the second component of the log-likelihood (penalty term)
associated with the random effects (frailty terms) in a mixed-effects model.
}
\details{
The penalty is derived from the Gaussian assumption on the random effects and
involves both the dimension of the random effects vector and the variance parameter \code{theta}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_G_function.R
\name{create_G_function}
\alias{create_G_function}
\title{Create Step Function for Estimated Survival (G)}
\usage{
create_G_function(unique_times, G_vals)
}
\arguments{
\item{unique_times}{A numeric vector of increasing unique times (e.g., censoring times).}

\item{G_vals}{A numeric vector of the same length as \code{unique_times}, corresponding to estimated values of the survival function at each time.}
}
\value{
A function \code{G(t)} that can be evaluated on a numeric vector of time points.
}
\description{
This function creates a right-continuous step function (vectorized) based on
estimated values of the censoring survival function at given unique times.
}
\details{
The resulting function can be used to evaluate \eqn{G(t)} for any vector of time points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_data.R
\name{simulate_data}
\alias{simulate_data}
\title{Simulate clustered competing risks data with shared frailty}
\usage{
simulate_data(
  G,
  Z = NULL,
  prop = 0.6,
  beta = NULL,
  theta = 0.5,
  cens = FALSE,
  pcens = 0.25,
  tau = 0.5
)
}
\arguments{
\item{G}{A vector of group or cluster identifiers (length \code{N}). Each value indicates which cluster the individual belongs to.}

\item{Z}{A matrix of covariates (dimensions \code{N x p}). Can be set to \code{NULL} if no covariates are used.}

\item{prop}{Proportion of individuals susceptible to cause 1 (default: \code{0.6}).}

\item{beta}{A numeric vector of regression coefficients, one per covariate (length \code{p}).}

\item{theta}{Variance of the shared frailty term for event times (cause 1).}

\item{cens}{Logical, indicating whether censoring should be simulated (default: \code{FALSE}).}

\item{pcens}{Target censoring proportion (default: \code{0.25}).}

\item{tau}{Variance of the shared frailty term for censoring times (default: \code{0}).}
}
\value{
A \code{data.frame} with one row per individual and at least the columns:
\describe{
\item{times}{Observed follow-up time.}
\item{status}{Event indicator: \code{0} for censoring, \code{1} for cause 1,
\code{2} for cause 2.}
\item{clusters}{Cluster identifier.}
}
If covariates are simulated, additional numeric covariate columns are included.
}
\description{
This function simulates clustered time-to-event data under a competing risks framework
with shared frailty and the possibility of random censoring. Cause 1 is modeled through
an inversion method of the subdistribution function, and cause 2 is introduced when inversion fails.
}
\examples{
\donttest{
n_cov <- 1
n_cluster <- 5
n_per_cluster <- 100
n <- n_cluster * n_per_cluster

G <- rep(1:n_cluster, each = n_per_cluster)
Z <- matrix(runif(n * n_cov), ncol = n_cov)

df <- simulate_data(
  G = G,
  Z = Z,
  prop = 0.6,
  beta = c(0.5),
  theta = 0.01,
  cens = TRUE,
  pcens = 0.25,
  tau = 0.01
)

head(df)
table(df$status)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Reml_CompRisk_frailty.R
\name{Reml_CompRisk_frailty}
\alias{Reml_CompRisk_frailty}
\title{REML Estimation for Cause 1 in a Competing Risks Model with Shared Frailty}
\usage{
Reml_CompRisk_frailty(
  data,
  cluster_censoring = FALSE,
  max_iter = 300,
  tol = 1e-06,
  threshold = 1e-05
)
}
\arguments{
\item{data}{A data frame with at least the following columns:
\describe{
\item{times}{Observed times to event or censoring.}
\item{status}{Event type: 0 = censored, 1 = cause of interest, >1 = other competing risks.}
\item{clusters}{Cluster membership (e.g., centers or subjects).}
\item{covariates}{(Optional) Columns from the 4th onward are used as covariates.}
}}

\item{cluster_censoring}{Logical. If \code{TRUE}, accounts for center-specific censoring using a frailty model on censoring times. Default is \code{FALSE}.}

\item{max_iter}{Maximum number of iterations for the Newton-Raphson algorithm (default = 300).}

\item{tol}{Convergence tolerance for the likelihood and frailty variance (default = 1e-6).}

\item{threshold}{Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).}
}
\value{
A list with:
\describe{
\item{beta}{Estimated regression coefficients for the cause 1 hazard.}
\item{u}{Estimated cluster-specific random effects (frailties).}
\item{theta}{Estimated frailty variance.}
\item{p_value}{P-value testing the null hypothesis \eqn{H_0: \theta = 0}.}
}
}
\description{
Fits a cause-specific hazard model for cause 1 in a competing risks framework using restricted maximum likelihood (REML) with a shared frailty term for clusters.
}
\details{
The function fits a proportional hazards model for cause 1, accounting for unobserved heterogeneity via a shared frailty term on clusters. When \code{cluster_censoring = TRUE}, a frailty-adjusted survival curve is used to correct for informative censoring using either a Kaplan-Meier or Nelson-Aalen-based estimator.

If the estimated frailty variance \eqn{\theta} becomes smaller than the provided \code{threshold}, the model reverts to a standard Cox model for cause 1 using \code{\link{Ml_CompRisk}}.
}
\examples{
\donttest{
data <- data.frame(
  times = c(4, 6, 10, 12, 3),
  status = c(1, 0, 2, 1, 0),
  clusters = c(1, 1, 2, 2, 3),
  x1 = rnorm(5),
  x2 = sample(0:1, 5, replace = TRUE)
)
result <- Reml_CompRisk_frailty(data)
result$beta
result$p_value
}

}
\seealso{
\code{\link{Ml_CompRisk}}, \code{\link{Reml_Cox_frailty}}, \code{\link{logLikelihood_1}}, \code{\link{compute_M_optimized}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLikelihood_1.R
\name{logLikelihood_1}
\alias{logLikelihood_1}
\title{Log-Likelihood Contribution for Cause 1}
\usage{
logLikelihood_1(status, M, W)
}
\arguments{
\item{status}{A numeric vector indicating the event type:
\code{0} for censored, \code{1} for cause 1, \code{2} for other causes.}

\item{M}{A numeric matrix (usually computed with \code{\link{compute_M_optimized}}), representing
IPCW weights between individuals.}

\item{W}{A sparse matrix (e.g., of class \code{dgCMatrix} from the \pkg{Matrix} package)
representing estimated weights or hazards.}
}
\value{
A numeric scalar: the log-likelihood contribution from all individuals who failed from cause 1.
}
\description{
Computes the contribution to the log-likelihood from individuals who experienced event type 1,
using a weighting matrix \code{W} and an IPCW matrix \code{M}.
}
\details{
For each individual \code{i} such that \code{status[i] == 1}, the function computes:
\deqn{
\log W_{ii} - \log\left(\sum_j M_{ij} W_{ij}\right)
}
and sums this across all such individuals.
}
\seealso{
\code{\link[=compute_M_optimized]{compute_M_optimized()}}, \link[Matrix:Matrix-class]{Matrix::Matrix}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Reml_Cox_frailty.R
\name{Reml_Cox_frailty}
\alias{Reml_Cox_frailty}
\title{Cox Model with Random Frailty (REML Estimation)}
\usage{
Reml_Cox_frailty(data, max_iter = 300, tol = 1e-06, threshold = 1e-05)
}
\arguments{
\item{data}{A data frame containing:
\itemize{
\item \code{times}: observed survival or censoring times,
\item \code{status}: event indicator (1 = event, 0 = censored),
\item \code{clusters}: cluster identifiers (e.g., center, group),
\item covariates from column 4 onward.
}}

\item{max_iter}{Maximum number of Newton-Raphson iterations (default = 300).}

\item{tol}{Tolerance for convergence (default = 1e-6).}

\item{threshold}{Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).}
}
\value{
A list with the following elements:
\describe{
\item{beta}{Estimated fixed effects (regression coefficients).}
\item{u}{Estimated random effects (frailties) for each cluster.}
\item{theta}{Estimated variance of the random effects.}
\item{p_value}{P-value for testing the null hypothesis \eqn{H_0: \theta = 0} (no frailty).}
}
}
\description{
Estimates the regression coefficients, random effects, and frailty variance \eqn{\theta} in a Cox proportional hazards model with shared frailty using Restricted Maximum Likelihood (REML).
}
\details{
The function uses a REML-based Newton-Raphson approach to estimate the parameters.
A penalized likelihood including a log-likelihood contribution for the frailty variance \eqn{\theta} is maximized.

The hypothesis test of \eqn{H_0: \theta = 0} is based on a Wald-type statistic. A high p-value (e.g. > 0.05) may indicate that the random effects (frailties) are negligible.
}
\seealso{
\code{\link{Ml_Cox}}, \code{\link{logLikelihood_1}}, \code{\link{logLikelihood_2}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_data_format.R
\name{check_data_format}
\alias{check_data_format}
\title{Check the Format of a Data Frame for Multicentre Competing Risks with Frailty}
\usage{
check_data_format(df)
}
\arguments{
\item{df}{A data frame to be checked.}
}
\value{
\code{TRUE} if \code{df} has the expected structure.
Otherwise, the function stops with an error describing the formatting issue.
}
\description{
Validates whether the submitted data frame is correctly structured for the analysis
of multicentre competing risks with frailty.
}
\examples{
\donttest{
n_cov = 1
n_cluster = 5
n_per_cluster = 100
n = n_cluster * n_per_cluster
a1 = 0
b1 = 1
G <- rep(1:n_cluster, each = n_per_cluster)
Z <- matrix(runif(n * n_cov, a1, b1), ncol = n_cov)
df = simulate_data(
  G,
  Z,
  prop = 0.6,
  beta = c(0),
  theta = 0.01,
  cens = TRUE,
  pcens = 0.25,
  tau = 0
)
check_data_format(df)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Parameters_estimation.R
\name{Parameters_estimation}
\alias{Parameters_estimation}
\title{Unified Interface for Parameter Estimation in (Competing Risks) Frailty Models}
\usage{
Parameters_estimation(
  data,
  method = "CompRisk_frailty",
  cluster_censoring = FALSE,
  max_iter = 300,
  tol = 1e-06,
  threshold = 1e-06
)
}
\arguments{
\item{data}{A data frame containing at least the following columns:
\describe{
\item{times}{Event or censoring times.}
\item{status}{Event indicator: 0 = censored, 1 = event of interest, >1 = other types (for competing risks).}
\item{clusters}{Cluster/group variable (e.g., center ID).}
\item{covariates}{(Optional) Additional covariates from the 4th column onward.}
}}

\item{method}{Character string indicating the estimation method. One of:
\itemize{
\item \code{"CompRisk_frailty"} (default): Competing risks model for cause 1 with shared frailty.
\item \code{"CompRisk"}: Competing risks model for cause 1 without frailty.
\item \code{"Cox_frailty"}: Cox proportional hazards model with shared frailty.
\item \code{"Cox"}: Standard Cox proportional hazards model.
}}

\item{cluster_censoring}{Logical. If \code{TRUE}, adjusts for cluster-specific censoring. Only applicable when \code{method = "CompRisk_frailty"} (default = \code{FALSE}).}

\item{max_iter}{Maximum number of iterations for the selected model estimation (default = 300).}

\item{tol}{Convergence tolerance (default = 1e-6).}

\item{threshold}{Lower bound for the frailty variance parameter \eqn{\theta}. If the estimated value falls below this threshold, frailty is considered negligible (default = 1e-5).}
}
\value{
A named list whose structure depends on \code{method}.
For \code{"CompRisk"} and \code{"Cox"}, the returned list contains:
\describe{
\item{beta}{Numeric vector of estimated regression coefficients.}
}
For \code{"CompRisk_frailty"} and \code{"Cox_frailty"}, the returned list may contain:
\describe{
\item{beta}{Numeric vector of estimated fixed-effect coefficients.}
\item{u}{Numeric vector of estimated cluster-specific frailties.}
\item{theta}{Estimated frailty variance parameter.}
\item{p_value}{P-value for testing the null hypothesis of no frailty effect.}
}
Returns \code{NULL} if estimation fails.
}
\description{
Provides a unified wrapper to estimate model parameters under four different modeling strategies:
\itemize{
\item Competing risks with shared frailty for cause 1 (\code{"CompRisk_frailty"})
\item Competing risks without frailty for cause 1 (\code{"CompRisk"})
\item Standard Cox model with shared frailty (\code{"Cox_frailty"})
\item Standard Cox model without frailty (\code{"Cox"})
}
}
\details{
This wrapper allows seamless switching between various modeling frameworks. It automatically calls:
\itemize{
\item \code{\link{Reml_CompRisk_frailty}} for \code{"CompRisk_frailty"}
\item \code{\link{Ml_CompRisk}} for \code{"CompRisk"}
\item \code{\link{Reml_Cox_frailty}} for \code{"Cox_frailty"}
\item \code{\link{Ml_Cox}} for \code{"Cox"}
}

Data format is validated via the helper function \code{\link{check_data_format}}. An error is raised if input format is invalid or if \code{cluster_censoring} is used with an incompatible method.
}
\examples{
\donttest{
data <- data.frame(
  times = c(4, 6, 10, 12, 3),
  status = c(1, 0, 2, 1, 0),
  clusters = c(1, 1, 2, 2, 3),
  x1 = rnorm(5),
  x2 = sample(0:1, 5, replace = TRUE)
)
result <- Parameters_estimation(data, method = "CompRisk_frailty")
result$beta
}

}
\seealso{
\code{\link{Reml_CompRisk_frailty}}, \code{\link{Ml_CompRisk}}, \code{\link{Reml_Cox_frailty}}, \code{\link{Ml_Cox}}, \code{\link{check_data_format}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ml_Cox.R
\name{Ml_Cox}
\alias{Ml_Cox}
\title{Maximum Likelihood Estimation for the Cox Model (Optimized)}
\usage{
Ml_Cox(data, max_iter = 100, tol = 1e-06)
}
\arguments{
\item{data}{A data frame with columns: \code{times}, \code{status}, and covariates starting from the 4th column.}

\item{max_iter}{Maximum number of iterations (default = 100).}

\item{tol}{Convergence tolerance for the Euclidean norm (default = 1e-6).}
}
\value{
A named list with one element:
\describe{
\item{beta}{A numeric vector of estimated regression coefficients in the
Cox proportional hazards model. Each coefficient represents the log hazard
ratio associated with the corresponding covariate.}
}
}
\description{
This function computes the maximum likelihood estimates of the regression coefficients in the Cox proportional hazards model using Newton-Raphson iterations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ml_CompRisk.R
\name{Ml_CompRisk}
\alias{Ml_CompRisk}
\title{Maximum Likelihood Estimation for Cause 1 in Competing Risks}
\usage{
Ml_CompRisk(data, max_iter = 100, tol = 1e-06)
}
\arguments{
\item{data}{A data frame containing:
\describe{
\item{times}{Observed survival or censoring times.}
\item{status}{Event indicator: 0 = censored, 1 = cause 1 (event of interest), 2+ = other causes.}
\item{Covariates}{All columns from the 4th onward are considered as covariates.}
}}

\item{max_iter}{Maximum number of Newton-Raphson iterations (default = 100).}

\item{tol}{Convergence tolerance for the change in parameter estimates (default = 1e-6).}
}
\value{
A named list with one element:
\describe{
\item{beta}{A numeric vector of estimated regression coefficients for the
subdistribution hazard of cause 1. Each coefficient represents the log
effect of the corresponding covariate on the subdistribution hazard.}
}
}
\description{
Performs maximum likelihood estimation (MLE) for the effect of covariates on the cause-specific hazard function of cause 1 in a competing risks framework.
}
\details{
The function fits a Cox-type model for the subdistribution hazard of cause 1, treating all other causes (including censoring) as censored (i.e., status != 1 becomes 0).
It uses a Newton-Raphson procedure to maximize the partial likelihood.

The design matrix is reordered internally by event time to facilitate risk set computation.
}
\examples{
\donttest{
data <- data.frame(
 times = c(0.099, 0.006, 0.260, 0.151, 0.262,
           0.019, 0.026, 0.241, 0.017, 0.195,
           0.007, 0.057, 0.241, 0.132, 0.044,
           0.242, 0.416, 0.096, 0.172, 0.015),
 status = c(1, 0, 1, 1, 0,
            0, 0, 0, 0, 0,
            0, 0, 0, 0, 0,
            0, 0, 0, 0, 0),
 clusters = c(1,1,1,1,1,
              1,1,1,1,1,
             2,2,2,2,2,
              2,2,2,2,2),
 V1 = c(0.720, 0.289, 0.382, 0.995, 0.045,
        0.048, 0.042, 0.810, 0.964, 0.781,
        0.443, 0.303, 0.902, 0.596, 0.462,
        0.521, 0.656, 0.164, 0.261, 0.635)
)
  beta_hat <- Ml_CompRisk(data)
}

}
\seealso{
\code{\link{Reml_Cox_frailty}}, \code{\link{Ml_Cox}}, \code{\link{logLikelihood_1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_M_optimized.R
\name{compute_M_optimized}
\alias{compute_M_optimized}
\title{Compute the Weight Matrix M (Optimized)}
\usage{
compute_M_optimized(times, status, f, u_bis, clusters)
}
\arguments{
\item{times}{A numeric vector of observed times (events or censoring).}

\item{status}{A numeric vector of status indicators (0 = censored, 1 or 2 = cause of failure).}

\item{f}{A string, either \code{"KaplanMeier_Censoring_vectorized"} or \code{"Nelson_Censoring_vectorized"},
specifying the method to estimate the censoring distribution.}

\item{u_bis}{A numeric vector of cluster-level random effects (used only if \code{f == "Nelson_Censoring_vectorized"}).}

\item{clusters}{A vector indicating cluster membership for each subject (same length as \code{times}).}
}
\value{
A square matrix \code{M} of size N x N, where \code{M[i, j]} is the IPCW weight between subject i and subject j.
}
\description{
This function computes the matrix of inverse probability of censoring weights (IPCW),
used in multicenter competing risks models with frailty. It supports both Kaplan-Meier and Nelson-Aalen estimators
for the censoring distribution.
}
\details{
\itemize{
\item If \code{f == "KaplanMeier_Censoring_vectorized"}, then \code{u_bis} is ignored.
\item The weight matrix accounts for pairwise contributions, depending on event times and causes.
}
}
\seealso{
\code{\link[=KaplanMeier_Censoring_vectorized]{KaplanMeier_Censoring_vectorized()}}, \code{\link[=Nelson_Censoring_vectorized]{Nelson_Censoring_vectorized()}}, \code{\link[=create_G_function]{create_G_function()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KaplanMeier_Censoring_vectorized.R
\name{KaplanMeier_Censoring_vectorized}
\alias{KaplanMeier_Censoring_vectorized}
\title{Kaplan-Meier Estimator for Censoring Distribution (Vectorized)}
\usage{
KaplanMeier_Censoring_vectorized(times, status, unique_times)
}
\arguments{
\item{times}{A numeric vector of observed times.}

\item{status}{A numeric vector of status indicators (0 = censored, >0 = event).}

\item{unique_times}{A numeric vector of time points at which to evaluate the estimator.}
}
\value{
A numeric vector of estimated survival probabilities at each unique time.
}
\description{
Computes the Kaplan-Meier estimator of the censoring distribution at given time points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Nelson_Censoring_vectorized.R
\name{Nelson_Censoring_vectorized}
\alias{Nelson_Censoring_vectorized}
\title{Nelson-Aalen Estimator of Censoring Distribution (Vectorized)}
\usage{
Nelson_Censoring_vectorized(times, unique_times, status, clusters, u_bis)
}
\arguments{
\item{times}{A numeric vector of observed times.}

\item{unique_times}{A numeric vector of distinct, sorted time points.}

\item{status}{A numeric vector where 0 indicates censoring.}

\item{clusters}{A vector indicating the cluster/group for each observation.}

\item{u_bis}{A numeric vector of cluster-specific random effects (e.g., frailty values).}
}
\value{
A numeric vector of estimated censoring survival probabilities for each individual.
}
\description{
Computes the estimated censoring survival probabilities using a Nelson-Aalen-type estimator,
accounting for shared frailty through a cluster-specific random effect.
}
